/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
# include <sys/time.h>

# include "TypeDef.h"
# include "VectorManip.h"
# include "Structures.h"
# include "Constants.h"
# include "IOUtils.h"
# include "MathStructures.h"
# include "MathUtils.h"
# include "time.h"
# include "MinMax.h"
# include "error.h"
# include "normalize.h"

void printMatrix(Matrix &Mat)
{
	int Size1, Size2;
	Get2DVectorSize(Mat, Size1, Size2, "Mat in printMatrix");
	for (int i=0;i<Size1;i++)
	{
                for (int j=0;j<Size2;j++)
                {
                        cout <<setiosflags(ios::left)<<setw(10)<<setprecision(5)<<Mat[i][j]<<"\t";
		}
                cout <<endl;
	}
	cout <<endl;
}

void printMatrix(Real **Mat, int Size1, int Size2)
{
	for (int i=0;i<Size1;i++)
	{
                for (int j=0;j<Size2;j++)
                {
                        cout <<setiosflags(ios::left)<<setw(10)<<setprecision(5)<<Mat[i][j]<<"\t";
		}
                cout <<endl;
	}
	cout <<endl;
}

string vectorToStr(VectorStruct &v)
{
        return "("+RealToStr(v.x)+", "+RealToStr(v.y)+", "+RealToStr(v.z)+")";
}

template <class Matrix_T>
void createMatrix(Matrix_T &Matrix, int Size1, int Size2)
{
	int i;
	vector<Real> v;
	for (i=0; i<Size2; i++) v.push_back(0);
	for (i=0; i<Size1; i++) Matrix.push_back(v);
}

template <class Matrix_T>
void deleteMatrix(Matrix_T &Matrix)
{
	int i, Size=Matrix.size();
	for (i=0; i<Size; i++) Matrix.pop_back();
}

bool isEven(int x)
{
        if (x%2==0) return true;
        else return false;
}

bool isOdd(int x)
{
        if (x%2==0) return false;
        else return true;
}

bool isEqual(Real x, Real y, Real d)
{
	if (abs(x-y)>d) return false;
	return true;
}

bool isEqual(Real x, Real y)
{
	Real d=1.0e-6;
	return isEqual(x, y, d);
}

bool vectorEqual(vector<Real> &v1, vector<Real> &v2, Real d)
{
	int Size1, Size2;

	Size1=v1.size();
	Size2=v2.size();
	if (Size1!=Size2) return false;

	for (int i=0;i<Size1;i++)
	{
		if (abs(v1[i]-v2[i])>d) 
		{
			return false;
		}
	}
	return true;
}

bool vectorEqual(vector<int> &v1, vector<int> &v2)
{
	int Size1, Size2;

	Size1=v1.size();
	Size2=v2.size();
	if (Size1!=Size2) return false;

	for (int i=0;i<Size1;i++)
	{
		if (v1[i]!=v2[i]) 
		{
			return false;
		}
	}
	return true;
}

bool matrixEqual(Matrix &mat1, Matrix &mat2)
{
	int Size1a, Size1b;
	int Size2a, Size2b;
	Real d=1.0e-6;

	Get2DVectorSize(mat1, Size1a, Size1b);
	Get2DVectorSize(mat2, Size2a, Size2b);

	if (Size1a!=Size2a) return false;
	if (Size1b!=Size2b) return false;

	for (int i=0;i<Size1a;i++)
	{
		for (int j=0;j<Size1b;j++)
		{
			if (abs(mat1[i][j]-mat2[i][j])>d) return false;
		}
	}
	return true;
}

bool isZero(Vector &v)
{
	int Size=v.size();

	if (Size==0) return true;
	for (int i=0;i<Size;i++)
	{
		if (v[i]!=0) return false;
	}
	return true;
}

bool isPositiveDefinite(Matrix &mat)
{
        //This does not actually determine if the matrix is positive definite.
        int Size=mat.size();
        for (int i=0;i<Size;i++)
        {
                if (mat[i][i]<0) return false;
        }
        return true;
}

bool quadraticEquation(Real a, Real b, Real c, Real &low, Real &high)
{
	Real sq;
	sq=b*b-4.0*a*c;
	if (sq<0) return false;
	low=(-b-sqrt(sq))/(2.0*a);
	high=(-b+sqrt(sq))/(2.0*a);
	return true;
}

/*
void CubicEquation(Real a, Real b, Real c, Real d, Real &x1, Real &x2, Real &x3)
{
	//WARNING: not tested not complete probably does not work
	//Solves equations of the form a*x^3+b*x^2+c*x+d=0
	//From en.wikipedia.org/wiki/Cubic_function
	//Fix later to find complex roots

	Real sqrtTerm, cubeTerm1, cubeTerm2;

	sqrtTerm=(2.0*b*b*b-9.0*a*b*c+27.0*a*a*d);
	sqrtTerm=sqrtTerm*sqrtTerm;
	sqrtTerm-=4.0*(b*b-3.0*a*c)*(b*b-3.0*a*c)*(b*b-3.0*a*c);
	if (sqrtTerm>0) sqrtTerm=sqrt(sqrtTerm);
	{
		error("Cannot solve cubic equation", __LINE__, __FILE__);
	}

	cubeTerm1=0.5*(2.0*b*b*b-9.0*a*b*c+27.0*a*a*d+sqrtTerm);
	cubeTerm2=0.5*(2.0*b*b*b-9.0*a*b*c+27.0*a*a*d-sqrtTerm);

	cubeTerm1=power(cubeTerm1, 1.0/3.0);
	cubeTerm2=power(cubeTerm2, 1.0/3.0);

	x1=-(b+cubeTerm1+cubeTerm2)/(3.0*a);
	x2=-(b-cubeTerm1-cubeTerm2)/(3.0*a);
}
*/

Real solveTrig(Real a, Real b, Real c)
{
        //Solves a*cos(theta)+b*sin(theta)=c for theta
        //a*cos(theta)+b*sin(theta)=d*sin(theta+phi)
        //d*sin(theta+phi)=d*cos(theta)*sin(phi)+d*sin(theta)*cos(phi)
        //a=d*sin(phi)  b=d*cos(phi)    a/b=tan(phi)
        Real d, phi, theta;
        phi=atan(a/b);
        d=sqrt(a*a+b*b);
        theta=asin(c/d)-phi;   //asin(c/d)+/-d 
        return theta;
}

void solveTrig(Real a, Real b, Real c, Real &theta1, Real &theta2)
{
        //Solves a*cos(theta)+b*sin(theta)=c for theta
        //a*cos(theta)+b*sin(theta)=d*sin(theta+phi)
        //d*sin(theta+phi)=d*cos(theta)*sin(phi)+d*sin(theta)*cos(phi)
        //a=d*sin(phi)  b=d*cos(phi)    a/b=tan(phi)
        Real c1, c2, d, phi;
        phi=atan(a/b);
        d=sqrt(a*a+b*b);
        theta1=asin(c/d)-phi;   //asin(c/d)+/-d 
	theta2=pi-asin(c/d)-phi;
	c1=a*cos(theta1)+b*sin(theta1);
	c2=a*cos(theta2)+b*sin(theta2);

	if (!isEqual(c, c1))
	{
		theta1+=pi;
		c1=a*cos(theta1)+b*sin(theta1);
		if (!isEqual(c, c1))
		{
			cout <<"ERROR: a= "<<a<<" b= "<<b<<" c= "<<c<<" theta1= "
			<<theta1<<" c1= "<<c1<<endl;
			endProgram(__LINE__, __FILE__);
		}
	}
	if (!isEqual(c, c2))
	{
		theta2+=pi;
		c2=a*cos(theta2)+b*sin(theta2);
		if (!isEqual(c, c2))
		{
			cout <<"ERROR: a= "<<a<<" b= "<<b<<" c= "<<c<<" theta2= "<<theta2<<endl;
			endProgram(__LINE__, __FILE__);
		}
	}
}

Real power(Real x, Real y)
{
	//Returns x^y.  Good for when y is not an integer.
	return exp(y*log(x));
}

VectorStruct VectorStruct::operator+ (VectorStruct v)
{
	VectorStruct temp;
	temp.x=x+v.x;
	temp.y=y+v.y;
	temp.z=z+v.z;
	return (temp);
}

VectorStruct VectorStruct::operator- (VectorStruct v)
{
	VectorStruct temp;
	temp.x=x-v.x;
	temp.y=y-v.y;
	temp.z=z-v.z;
	return (temp);
}

VectorStruct VectorStruct::operator* (Real scalar)
{
	VectorStruct temp;
	temp.x=x*scalar;
	temp.y=y*scalar;
	temp.z=z*scalar;
	return (temp);
}

Real VectorStruct::operator* (VectorStruct v)
{
	Real scalar;
	scalar=v.x*x+v.y*y+v.z*z;
	return scalar;
}

VectorStruct VectorStruct::operator/ (Real scalar)
{
	VectorStruct temp;
	temp.x=x/scalar;
	temp.y=y/scalar;
	temp.z=z/scalar;
	return (temp);
}

Real vectorLength(VectorStruct &v)
{
	return sqrt(v.x*v.x+v.y*v.y+v.z*v.z);
}

Real calcLength(Vector &v)
{
	int Size=v.size();
	Real sum=0;

	for (int i=0;i<Size;i++)
	{
		sum+=v[i]*v[i];
	}
	return sqrt(sum);
}

void vectorNormalize(VectorStruct &v)
{
        Real length=vectorLength(v);
        v=v/length;
}

void vectorNormalize(VectorStruct &v1, VectorStruct &v2)
{
        Real length=vectorLength(v1);
        v2=v1/length;
}

VectorStruct setLength(VectorStruct &v, Real length)
{
	Real dist=vectorLength(v);
	VectorStruct vOut;
	vOut=v*(length/dist);
	return vOut;
}

void setLength(Vector &v, Real length)
{
	int Size=v.size();
	Real dist=calcLength(v);

	for (int i=0;i<Size;i++) v[i]*=(length/dist);
}

VectorStruct calcCrossProduct(VectorStruct &v1, VectorStruct &v2)
{
	VectorStruct product;

	product.x=v1.y*v2.z-v1.z*v2.y;
	product.y=v2.x*v1.z-v2.z*v1.x;
	product.z=v1.x*v2.y-v1.y*v2.x;

	return product;
}

Vector calcCrossProduct(Vector &v1, Vector &v2)
{
	int Size1, Size2;
	Vector product;

	Size1=v1.size();
	Size2=v2.size();

	if (Size1!=3 || Size2!=3)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	product=v1;

	product[X]=v1[Y]*v2[Z]-v1[Z]*v2[Y];
	product[Y]=v2[X]*v1[Z]-v2[Z]*v1[X];
	product[Z]=v1[X]*v2[Y]-v1[Y]*v2[X];

	return product;
}

Real calcDotProduct(Vector &v1, Vector &v2)
{
	int Size1=v1.size();
	int Size2=v2.size();
	Real dp=0;

	if (Size1!=Size2)
	{
		string errorStr;
		errorStr="Cannot calculate dotproduct Size1= "+IntToStr(Size1);
		errorStr+=" Size2= "+IntToStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	for (int i=0;i<Size1;i++) dp+=v1[i]*v2[i];
	
	return dp;
}

Real calcDotProduct(VectorStruct &v1, Vector &v2)
{
	int Size=v2.size();

	if (Size!=3)
	{
		string errorStr;
		errorStr="Trying to calculate dotproduct but vector size is ";
		errorStr+=IntToStr(Size);
		error(errorStr, __LINE__, __FILE__);
	}

	return v1.x*v2[X]+v1.y*v2[Y]+v1.z*v2[Z];
}

Real calcAngle(VectorStruct &v1, VectorStruct &v2)
{
	Real angle;
        Real dotProduct=v1*v2;
        angle=acos(dotProduct/(vectorLength(v1)*vectorLength(v2)));
	return angle;
}

Real calcAngle(Vector &v1, Vector &v2)
{
	int Size1=v1.size();
	int Size2=v2.size();
	Real dp, length1, length2;

	if (Size1!=Size2 || Size1==0)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}
	dp=calcDotProduct(v1, v2);
	length1=calcLength(v1);
	length2=calcLength(v2);
	return acos(dp/(length1*length2));
}

Real calcDistance(Real x1, Real y1, Real z1, Real x2, Real y2, Real z2)
{
	Real dx, dy, dz;

	dx=x2-x1;
	dy=y2-y1;
	dz=z2-z1;

	return sqrt(dx*dx+dy*dy+dz*dz);
}

Real calcDistance(Vector &v1, Vector &v2)
{
	int Size1=v1.size();
	int Size2=v2.size();
	Real sum=0;

	if (Size1!=Size2 || Size1==0)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	for (int i=0;i<Size1;i++)
	{
		sum+=(v1[i]-v2[i])*(v1[i]-v2[i]);
	}
	return sum;
}

Real calcProjection(VectorStruct &v1, VectorStruct &v2)
{
	Real angle=calcAngle(v1, v2);
	Real length=vectorLength(v1);
	return length*cos(angle);
}

Real calcDeltaAngle(Real angle1, Real angle2)
{
        int mod;
        Real dAngle;
        mod=int(abs(angle2-angle1)/(2.0*pi));
        if (angle2>angle1) dAngle=angle2-angle1-2.0*pi*Real(mod);
        else if (angle2<angle1) dAngle=abs(angle2-angle1+2.0*pi*Real(mod));
        else return 0;
        if (dAngle>pi) dAngle=2.0*pi-dAngle;
        return dAngle;
}

void calcEulerAngles(Real x, Real y, Real z, Real &theta, Real &phi)
{
        //Gives the Euler angles to rotate a vector the xyz coordinates given from the z-axis.
        theta=acos(-y/sqrt(1.0-z*z));
        phi=acos(z);
}

void calcPolarCoordinates(VectorStruct v, Real &ry, Real &rz)
{
	Real x, y;
	vectorNormalize(v);


	ry=acos(v.z);
	x=v.x/sin(ry);
	if (x>1.0) x=1.0;
	else if (x<-1.0) x=-1.0;
        rz=acos(x);
        y=sin(rz)*sin(ry);

        if (!isEqual(y, v.y))
        {
                ry=-ry;
                rz=asin(v.x/sin(ry));
        }
}

void polar2Cartesian(Real &x, Real &y, Real &z, Real &ry, Real &rz)
{
	x=cos(rz)*sin(ry);
	y=sin(rz)*sin(ry);
	z=cos(ry);
}

void polar2Cartesian(VectorStruct &v, Real &ry, Real &rz)
{
	polar2Cartesian(v.x, v.y, v.z, ry, rz);
}

template<class pos>
void rotateAroundX(pos &coord, Real theta)
{
        Real tempY, tempZ;
        tempY=coord.y*cos(theta)-coord.z*sin(theta);
        tempZ=coord.y*sin(theta)+coord.z*cos(theta);
        coord.y=tempY;
        coord.z=tempZ;
}

template<class pos>
void rotateAroundY(pos &coord, Real theta)
{
        Real tempX, tempZ;
        tempX=coord.x*cos(theta)-coord.z*sin(theta);
        tempZ=coord.x*sin(theta)+coord.z*cos(theta);
        coord.x=tempX;
        coord.z=tempZ;
}

template<class pos>
void rotateAroundZ(pos &coord, Real theta)
{
        Real tempX, tempY;
        tempX=coord.x*cos(theta)-coord.y*sin(theta);
        tempY=coord.x*sin(theta)+coord.y*cos(theta);
        coord.x=tempX;
        coord.y=tempY;
}

void calcRotationMatrixAroundX(Real theta, Matrix &rotationMatrix)
{
	int Size1, Size2=0;

	Size1=rotationMatrix.size();
	if (Size1!=0) Size2=rotationMatrix.size();

	if (Size1!=3 || Size2!=3) Safe2DAlloc(rotationMatrix, 3, 3, "rotMat");

	rotationMatrix[X][X]=1.0;	
	rotationMatrix[X][Y]=0;	
	rotationMatrix[X][Z]=0;
	
	rotationMatrix[Y][X]=0;	
	rotationMatrix[Y][Y]=cos(theta);	
	rotationMatrix[Y][Z]=-sin(theta);

	rotationMatrix[Z][X]=0;
	rotationMatrix[Z][Y]=sin(theta);
	rotationMatrix[Z][Z]=cos(theta);
}

void calcRotationMatrixAroundY(Real theta, Matrix &rotationMatrix)
{
	int Size1, Size2=0;

	Size1=rotationMatrix.size();
	if (Size1!=0) Size2=rotationMatrix.size();

	if (Size1!=3 || Size2!=3) Safe2DAlloc(rotationMatrix, 3, 3, "rotMat");

	rotationMatrix[X][X]=cos(theta);	
	rotationMatrix[X][Y]=0;	
	rotationMatrix[X][Z]=sin(theta);
	
	rotationMatrix[Y][X]=0;	
	rotationMatrix[Y][Y]=1.0;	
	rotationMatrix[Y][Z]=0;

	rotationMatrix[Z][X]=-sin(theta);
	rotationMatrix[Z][Y]=0;
	rotationMatrix[Z][Z]=cos(theta);
}

void calcRotationMatrixAroundZ(Real theta, Matrix &rotationMatrix)
{
	int Size1, Size2=0;

	Size1=rotationMatrix.size();
	if (Size1!=0) Size2=rotationMatrix.size();

	if (Size1!=3 || Size2!=3) Safe2DAlloc(rotationMatrix, 3, 3, "rotMat");

	rotationMatrix[X][X]=cos(theta);	
	rotationMatrix[X][Y]=-sin(theta);	
	rotationMatrix[X][Z]=0;
	
	rotationMatrix[Y][X]=sin(theta);	
	rotationMatrix[Y][Y]=cos(theta);	
	rotationMatrix[Y][Z]=0;

	rotationMatrix[Z][X]=0;
	rotationMatrix[Z][Y]=0;
	rotationMatrix[Z][Z]=1.0;
}

Real coth(Real x)
{
	Real exp2;
	exp2=exp(2.0*x);
	return (exp2+1.0)/(exp2-1.0);
}

Real csch(Real x)
{
	//cout <<"exp("<<x<<")= "<<exp(x)<<endl;
	return 2.0/(exp(x)-exp(-x));
}

Real calcBesselRecursive(long double r, int l)
{
        //Calculates the bessel function using a recursive formula. 
        //This can fail especially for small r.
        int i;
        vector<long double> j;
        long double invr;
        long double longi;

        j.resize(l+2);

        invr=1.0/r;
        j[0]=sin(r)*invr;
        j[1]=(j[0]-cos(r))*invr;

        longi=3.0;
        for (i=2;i<l+1;i++)
        {
                j[i]=j[i-1]*longi*invr-j[i-2];
                longi=longi+2.0;
        }

        if (abs(j[l])>1)
        {
                j[l]=0;
                cout <<"Bessel overflow\n";
        }

        return j[l];
}

Real calcBessel(Real r, int l, vector<long double> sterm)
{
        //Calculates the bessel function using Taylor expansion.  It can give
        //bad answers for large r.  So sometimes it switches to bessel2.
        int m;
	int Size=sterm.size();
        long double total;
        long double term;
        long double n;
        long double r2;
	long double longl, longm;

        term=1.0;
        total=0.0;
        r2=(long double)r*(long double)r;
        m=1;

        for (n=1.0;n<=l;n+=1.0)
        {
                term=term*r/(2.0*n+1.0);
        }

        while (abs(term)>0.0000001)
        {
                total+=term;
		if (m<Size) term=-term*sterm[m]*r2;
		else 
		{
			longl=(long double)(l);
			longm=(long double)(m);
			term=-term*r2*(longm+longl)/(2.0*(2.0*longm+2.0*longl+1.0)*(longm+longl)*longm);
		}
                m++;
        }

        if (abs(total)>1) total=calcBesselRecursive(r, l);

        return total;
}

void calcSTerm(int l, vector<long double> &sterm)
{
	//Used for calculating bessel functions.
	int maxM=150;
	long double longl, longm;

	longl=(long double)(l);
	SafeAlloc(sterm, maxM, "sterm");
	for (int m=1;m<maxM;m++)
	{
		longm=(long double)(m);
		sterm[m]=(longm+longl);
		sterm[m]/=(2.0*(2.0*longm+2.0*longl+1.0)*(longm+longl)*longm);
	}
}

Real calcBessel(Real r, int l)
{
	vector<long double> sterm;

	return calcBessel(r, l, sterm);
}

Real calcBesselI0(Real x)
{
	//en.wikipedia.org/wiki/Bessel_function
	//This will give the wrong result for large x.
	Real d, m, sum, term;

	d=1.0e-10;
	m=0;
	sum=1.0;
	term=1.0;
	while (true)
	{
		m+=1.0;
		term/=(m*m);
		term*=(x*x*0.25);
		if (isinf(term)) return sum;
		sum+=term;
		if (term<d) break;	
	}
	return sum;
}

Real calcLogBesselI0(Real x)
{
	Real d=-10, m=0, sum=0, term=0;
	Real cutoff=10.0;

	if (x==0) return 0;
	while (true)
	{
		m+=1.0;
		term-=2.0*log(m);
		term+=2.0*log(x)+log(0.25);
		if (abs(sum-term)<cutoff) sum=term+log(1.0+exp(sum-term));
		else if (sum-term<-cutoff) sum=term;
		if (term<d) break;
	}
	return sum;
}

void calcSphericalHarmonic(Real phi, Real theta, int lvalue, int mvalue, Real &realOut, Real &imagOut)
{
        //Evaluates a spherical harmonic.  Untested.

        int l;
        Real ctheta;
        Real stheta;
        Real stheta2;
        Real ylm;
        Matrix wlm;
        Vector pow;
        Real sign;
        long double longl;
        long double longm;
        vector<long double> sqrtm;
        vector<long double> invsqrtm;

	SafeAlloc(pow, lvalue, "pow");
	SafeAlloc(sqrtm, (long double)(0), lvalue+1, "sqrtm");
	SafeAlloc(invsqrtm, (long double)(0), lvalue+1, "invsqrtm");
	Safe2DAlloc(wlm, lvalue+1, lvalue+1, "wlm");

        for (int n=0;n<lvalue+1;n++)
        {
                pow[n]=0;
                sqrtm[n]=0;
                invsqrtm[n]=0;
                for (int m=0;m<lvalue+1;m++)
                {
                        wlm[n][m]=0;
                }
        }

        longl=1.0;
        wlm[0][0]=1.0;
        for (l=1;l<lvalue+1;l++)
        {
                wlm[l][l]=wlm[l-1][l-1]*(1.0-1.0/(2.0*longl));
                longl=longl+1.0;
        }

        longl=1.0;
        sign=-1.0;

        for (l=1;l<lvalue+1;l++)
        {
                wlm[l][l]=sign*sqrt((2*longl+1.0)*wlm[l][l]/(4.0*pi));
                longl=longl+1.0;
                sign=-sign;
        }

        wlm[0][0]=wlm[0][0]/sqrt(4.0*pi);

        pow[0]=1.0;

        longl=0;

        longm=0;

        for (int m=0;m<lvalue-1;m++)
        {
                sqrtm[m]=sqrt((longl+longm+2.0)*(longl-longm-1.0));
                invsqrtm[m]=1.0/sqrt((longl-longm)*(longl+longm+1.0));
                longm=longm+1.0;
        }
	stheta=sin(theta);
	ctheta=sin(theta);
        stheta2=stheta*stheta;

        if (l>0)
        {
                wlm[l][l-1]=-sqrt(2.0*longl)*ctheta*wlm[l][l];
        }

        for (int t=1;t<=lvalue;t++)
        {
                pow[t]=pow[t-1]*stheta;
        }

        longm=2.0*longl-2.0;
        for (int m=lvalue-2;m>-lvalue;m-=1)
        {
                wlm[lvalue][m]=-(sqrtm[m]*stheta2*wlm[l][m+2]+longm*ctheta*wlm[l][m+1])*invsqrtm[m];
                longm=longm-2.0;
        }


        ylm=wlm[lvalue][mvalue]*pow[mvalue];
        longm=phi*Real(mvalue);
        realOut=ylm*cos(longm);
        imagOut=ylm*sin(longm);
}

void calcFactorials(int maxFactorial, Vector &factorials)
{
	int Size=factorials.size();

	if (Size!=maxFactorial) 
	{
		SafeAlloc(factorials, maxFactorial, "factorials");
	}
	factorials[0]=1.0;

	for (int i=1;i<maxFactorial;i++)
	{
		factorials[i]=factorials[i-1]*Real(i);
	}
}

/*
Real calcWigner_dElement(int j, int m1, int m2, Real theta, Vector &factorials, Vector &cosPowers, Vector &sinPowers)
{
	//en.wikipedia.org/wiki/Wigner_D-matrix
	//Not tested may be wrong.
	int sMax, sMin;
	int maxFactorial, cosDegree, sinDegree;
	int factorialSize=factorials.size();
	int cosSize=cosPowers.size();
	int sinSize=sinPowers.size();
	Real factorialTerm, product;
	Real term, sum=0;

	product=1.0;
	sMax=min(j+m2, j+m1);
	sMin=max(0, m1-m2);
	maxFactorial=max(j+m1, j+m2);
	maxFactorial=max(maxFactorial, sMax);
	cosDegree=2*j+m2-m1;
	sinDegree=m1-m2+sMax;

	if (factorialSize<maxFactorial) calcFactorials(maxFactorial, factorials);
	if (cosSize<cosDegree) calcPowers(cos(theta*0.5), cosDegree, cosPowers);
	if (sinSize<sinDegree) calcPowers(sin(theta*0.5), sinDegree, sinPowers);

	factorialTerm=sqrt(factorials[j+m1]);
	factorialTerm*=sqrt(factorials[j-m1]);
	factorialTerm*=sqrt(factorials[j+m2]);
	factorialTerm*=sqrt(factorials[j-m2]);

	for (int s=sMin;s<=sMax;s++)
	{
		if (2*j+m2-m1-2*s>=0) term=cosPowers[2*j+m2-m1-2*s];
		else term=calcPower(cos(theta*0.5), 2*j+m2-m1-2*s);
		term*=sinPowers[m1-m2+2*s];
		term/=factorials[j+m2-s];
		term/=factorials[s];
		term/=factorials[m1-m2+s];
		term/=factorials[j+m1-s];
		if (isOdd(m1-m2+s)) term*=-1.0;
		sum+=term;
	}
	return sum*factorialTerm;
}

Real calcWigner_dElement(int j, int m1, int m2, Real theta)
{
	//en.wikipedia.org/wiki/Wigner_D-matrix
	Vector factorials, cosPowers, sinPowers;

	return calcWigner_dElement(j, m1, m2, theta, factorials, cosPowers, sinPowers);
}
*/

void initializeNegativeExpLookUp(LookUpStruct &lookUp, Real max, Real binSize)
{
	int nbin;
	Real x=0;

	lookUp.negativeExpMax=max;
	lookUp.negativeExpBin=binSize;
	lookUp.invNegativeExpBin=1.0/binSize;
	nbin=int(max/binSize)+1;
	SafeAlloc(lookUp.negativeExpLookUp, nbin, "expLookUp");
	SafeAlloc(lookUp.negativeExpDer, nbin, "expLookUp");
	for (int i=0;i<nbin;i++)
	{
		lookUp.negativeExpLookUp[i]=1.0/exp(x);
		lookUp.negativeExpDer[i]=(1.0/exp(x+binSize)-1.0/exp(x))/binSize;
		x+=binSize;
	}
}

void initializeNegativeExpLookUp(LookUpStruct &lookUp)
{
	lookUp.negativeExpMax=10.0;
	lookUp.negativeExpBin=1.0e-3;

	initializeNegativeExpLookUp(lookUp, lookUp.negativeExpMax, lookUp.negativeExpBin);
}

Real getNegativeExpLookUp(LookUpStruct &lookUp, Real x)
{
	int bin;
	int maxBin=lookUp.negativeExpLookUp.size();
	Real dx;

	x=abs(x);
	bin=int(x*lookUp.invNegativeExpBin);
	if (bin>=0 && bin<maxBin)
	{
		dx=x-Real(bin)*lookUp.negativeExpBin;
		//cout <<"x= "<<x<<" dx= "<<dx<<" bin= "<<bin<<"\t"
		//<<lookUp.negativeExpLookUp[bin]<<"\t"
		//<<lookUp.negativeExpDer[bin]<<endl;
		return lookUp.negativeExpLookUp[bin]+dx*lookUp.negativeExpDer[bin];
	}
	else return 1.0/exp(x);
}

void initializeExpLookUp(LookUpStruct &lookUp, Real max, Real binSize)
{
	int nbin;
	Real x=0;

	lookUp.expMax=max;
	lookUp.expBin=binSize;
	lookUp.invExpBin=1.0/binSize;

	nbin=int(max/binSize+0.5)+1;
	SafeAlloc(lookUp.expLookUp, nbin, "expLookUp");
	for (int i=0;i<nbin;i++)
	{
		lookUp.expLookUp[i]=exp(x);
		x+=binSize;
	}
}

void initializeExpLookUp(LookUpStruct &lookUp)
{
	lookUp.expMax=10.0;
	lookUp.expBin=1.0e-5;

	initializeExpLookUp(lookUp, lookUp.expMax, lookUp.expBin);
}

Real getExpLookUp(LookUpStruct &lookUp, Real x)
{
	int bin;
	int maxBin=lookUp.expLookUp.size();
	bin=int(x*lookUp.invExpBin+0.5);
	cout <<"x= "<<x<<" invExpBin= "<<lookUp.invExpBin<<" bin= "<<bin<<endl;
	if (bin>0 && bin<maxBin)
	{
		return lookUp.expLookUp[bin];
	}
	else return exp(x);
}

void initializeCosLookUp_Old(LookUpStruct &lookUp, Real binSize)
{
	int nbin;
	Real x=0;

	lookUp.cosBin=binSize;
	lookUp.invCosBin=1.0/binSize;

	nbin=int(2.0*pi/binSize);
	SafeAlloc(lookUp.cosLookUp, nbin, "cosLookUp");
	SafeAlloc(lookUp.cosDer, nbin, "cosDer");
	for (int i=0;i<nbin;i++)
	{
		lookUp.cosLookUp[i]=cos(x);
		lookUp.cosDer[i]=(cos(x+binSize)-cos(x))/binSize;
		x+=binSize;
	}
}

void initializeCosLookUp(LookUpStruct &lookUp, Real binSize)
{
	int nbin;
	Real x=0;
	Real x1, x2, x3;
	Real y1, y2, y3;
	Real a, b, c;

	cout <<"In initializeCosLookUp"<<endl;

	lookUp.cosBin=binSize;
	lookUp.invCosBin=1.0/binSize;

	nbin=int(2.0*pi/binSize);
	SafeAlloc(lookUp.cosLookUpA, nbin, "cosLookUp");
	SafeAlloc(lookUp.cosLookUpB, nbin, "cosLookUp");
	SafeAlloc(lookUp.cosLookUpC, nbin, "cosLookUp");
	for (int i=0;i<nbin;i++)
	{
		x1=x;
		x2=x+binSize;
		x3=x+2.0*binSize;
		y1=cos(x1);
		y2=cos(x2);
		y3=cos(x3);
		fitQuadratic(0, y1, binSize, y2, 2.0*binSize, y3, a, b, c);
		//cout <<"a= "<<a<<" b= "<<b<<" c= "<<c<<endl;
		lookUp.cosLookUpA[i]=a;
		lookUp.cosLookUpB[i]=b;
		lookUp.cosLookUpC[i]=c;
		x+=binSize;
	}
	cout <<"Leaving initializeCosLookUp"<<endl;
}

void initializeCosLookUp(LookUpStruct &lookUp)
{
	int nbin=100;
	lookUp.cosBin=2.0*pi/Real(nbin);

	initializeCosLookUp(lookUp, lookUp.cosBin);
}


void initializeSinLookUp_Old(LookUpStruct &lookUp, Real binSize)
{
	int nbin;
	Real x=0;

	lookUp.sinBin=binSize;
	lookUp.invSinBin=1.0/binSize;
	nbin=int(2.0*pi/binSize);
	SafeAlloc(lookUp.sinLookUp, nbin, "sinLookUp");
	SafeAlloc(lookUp.sinDer, nbin, "sinLookUp");
	for (int i=0;i<nbin;i++)
	{
		lookUp.sinLookUp[i]=sin(x);
		lookUp.sinDer[i]=(sin(x+binSize)-sin(x))/binSize;
		x+=binSize;
	}
}

void initializeSinLookUp(LookUpStruct &lookUp, Real binSize)
{
	int nbin;
	Real x=0;
	Real x1, x2, x3;
	Real y1, y2, y3;
	Real a, b, c;

	lookUp.sinBin=binSize;
	lookUp.invSinBin=1.0/binSize;

	nbin=int(2.0*pi/binSize);
	SafeAlloc(lookUp.sinLookUpA, nbin, "sinLookUp");
	SafeAlloc(lookUp.sinLookUpB, nbin, "sinLookUp");
	SafeAlloc(lookUp.sinLookUpC, nbin, "sinLookUp");
	for (int i=0;i<nbin;i++)
	{
		x1=x;
		x2=x+binSize;
		x3=x+2.0*binSize;
		y1=sin(x1);
		y2=sin(x2);
		y3=sin(x3);
		fitQuadratic(0, y1, binSize, y2, 2.0*binSize, y3, a, b, c);
		lookUp.sinLookUpA[i]=a;
		lookUp.sinLookUpB[i]=b;
		lookUp.sinLookUpC[i]=c;
		x+=binSize;
	}
}

void initializeSinLookUp(LookUpStruct &lookUp)
{
	int nbin=100;
	lookUp.sinBin=2.0*pi/Real(nbin);

	initializeSinLookUp(lookUp, lookUp.sinBin);
}

Real getSinLookUp(LookUpStruct &lookUp, Real x)
{
	int bin;
	int maxBin=lookUp.sinLookUp.size();

	while (x>2.0*pi) x-=2.0*pi;
	while (x<0) x+=2.0*pi;	
	bin=int(x*lookUp.invSinBin+0.5);
	if (bin>0 && bin<maxBin)
	{
		return lookUp.sinLookUp[bin];
	}
	else return sin(x);
}

Real getCosLookUp(LookUpStruct &lookUp, Real x)
{
	int bin;
	int maxBin=lookUp.cosLookUp.size();

	while (x>2.0*pi) x-=2.0*pi;
	while (x<0) x+=2.0*pi;	
	bin=int(x*lookUp.invCosBin+0.5);
	if (bin>0 && bin<maxBin)
	{
		return lookUp.cosLookUp[bin];
	}
	else return cos(x);
}

void getTrigLookUp_Old(LookUpStruct &lookUp, Real x, Real &cosx, Real &sinx)
{
	int bin;
	int maxCosBin=lookUp.cosLookUp.size();
	Real dx;
/*
	if (maxSinBin!=maxCosBin || maxSinBin==0)
	{
		string errorStr="maxSinBin= "+IntToStr(maxSinBin);
		errorStr+=" maxCosBin= "+IntToStr(maxCosBin);
		error(errorStr, __LINE__, __FILE__);
	}

	if (lookUp.sinBin!=lookUp.cosBin)
	{
		string errorStr="sinBin= "+RealToStr(lookUp.sinBin);
		errorStr+=" cosBin= "+RealToStr(lookUp.cosBin);
		error(errorStr, __LINE__, __FILE__);
	}
*/
	bin=int(abs(x)*lookUp.invSinBin);
	dx=abs(x)-Real(bin)*lookUp.cosBin;
	//cout <<"x= "<<x<<" dx= "<<dx<<endl;
	if (bin>=maxCosBin)
	{
		bin=bin%maxCosBin;
	}
	cosx=lookUp.cosLookUp[bin]+dx*lookUp.cosDer[bin];
	sinx=lookUp.sinLookUp[bin]+dx*lookUp.sinDer[bin];
	//cout <<"cosx= "<<cosx<<" cosLookUp= "<<lookUp.cosLookUp[bin]
	//<<" dx= "<<dx<<" cosDer= "<<lookUp.cosDer[bin]<<endl;
	if (x<0) sinx=-sinx;
}

void getTrigLookUp(LookUpStruct &lookUp, Real &x, Real &cosx, Real &sinx)
{
	int bin;
	int maxCosBin=lookUp.cosLookUpA.size();
	Real dx;

	//if (isnan(x) || isinf(x))
	//{
	//	error("x= "+toStr(x), __LINE__, __FILE__);
	//}
/*
	if (maxSinBin!=maxCosBin || maxSinBin==0)
	{
		string errorStr="maxSinBin= "+IntToStr(maxSinBin);
		errorStr+=" maxCosBin= "+IntToStr(maxCosBin);
		error(errorStr, __LINE__, __FILE__);
	}

	if (lookUp.sinBin!=lookUp.cosBin)
	{
		string errorStr="sinBin= "+RealToStr(lookUp.sinBin);
		errorStr+=" cosBin= "+RealToStr(lookUp.cosBin);
		error(errorStr, __LINE__, __FILE__);
	}
*/
	//cout <<"x= "<<x<<endl;
	bin=int(abs(x)*lookUp.invSinBin);
	//cout <<"invSinBin= "<<lookUp.invSinBin<<endl;
	//cout <<"cosBin= "<<lookUp.cosBin<<endl;
	//cout <<"bin= "<<bin<<endl;
	dx=abs(x)-Real(bin)*lookUp.cosBin;
	//cout <<"dx= "<<dx<<endl;
	if (bin>=maxCosBin)
	{
		bin=bin%maxCosBin;
	}
	cosx=lookUp.cosLookUpA[bin]*dx*dx+lookUp.cosLookUpB[bin]*dx+lookUp.cosLookUpC[bin];
	sinx=lookUp.sinLookUpA[bin]*dx*dx+lookUp.sinLookUpB[bin]*dx+lookUp.sinLookUpC[bin];

	//cout <<"cosx= "<<cosx<<" sinx= "<<sinx<<endl;
	//cout <<"cosx= "<<cosx<<" cosLookUp= "<<lookUp.cosLookUp[bin]
	//<<" dx= "<<dx<<" cosDer= "<<lookUp.cosDer[bin]<<endl;
	if (x<0) sinx=-sinx;
}

LookUpStruct& LookUpStruct::operator = (const LookUpStruct &lookUp)
{
	cosLookUp=lookUp.cosLookUp;
	cosDer=lookUp.cosDer;
	cosLookUpA=lookUp.cosLookUpA;
	cosLookUpB=lookUp.cosLookUpB;
	cosLookUpC=lookUp.cosLookUpC;
	sinLookUp=lookUp.sinLookUp;
	sinDer=lookUp.sinDer;
	sinLookUpA=lookUp.sinLookUpA;
	sinLookUpB=lookUp.sinLookUpB;
	sinLookUpC=lookUp.sinLookUpC;
	expLookUp=lookUp.expLookUp;
	negativeExpLookUp=lookUp.negativeExpLookUp;
	negativeExpDer=lookUp.negativeExpDer;
	cosBin=lookUp.cosBin;
	sinBin=lookUp.sinBin;
	expBin=lookUp.expBin;
	expMax=lookUp.expMax;
	negativeExpBin=lookUp.negativeExpBin;
	negativeExpMax=lookUp.negativeExpMax;
	invCosBin=lookUp.invCosBin;
	invSinBin=lookUp.invSinBin;
	invExpBin=lookUp.invExpBin;
	invNegativeExpBin=lookUp.invNegativeExpBin;
	return *this;
}

Real calcVolume(VectorStruct &a, VectorStruct &b, VectorStruct &c)
{
	VectorStruct cp;
	cp=calcCrossProduct(a, b);
	return c*cp;
}

void calcHistogram(Vector &v, Vector &histogram, Real binSize)
{
	int Size=v.size();
	int nbin, bin;
	Real max;
	
	max=findMax(v);
	cout <<"max= "<<max<<endl;
	nbin=int(max/binSize+0.5)+1;
	SafeAlloc(histogram, nbin, "histogram");

	for (int i=0;i<Size;i++)
	{
		bin=int(v[i]/binSize+0.5);
		histogram[bin]+=1.0;
	}
}

void calcHistogram(Vector &v, Vector &histogram, Real binSize, Real min, int nbin)
{
	int Size=v.size();
	int bin;
	
	SafeAlloc(histogram, nbin, "histogram");

	for (int i=0;i<Size;i++)
	{
		bin=int((v[i]-min)/binSize+0.5);
		if (bin>=0 && bin<nbin) histogram[bin]+=1.0;
	}
}

void calcHistogram(Vector &v, Vector &histogram, Real &binSize, Real &min)
{
	int Size=v.size();
	int nbin;
	Real max;

	nbin=int(sqrt(Size));
	min=findMin(v);
	max=findMax(v);
	binSize=(max-min)/Real(nbin);

	calcHistogram(v, histogram, binSize, min, nbin);
}

/*
void equation(long double **coefficient, long double variable[], int nequations)
{
	//Finds the solution to a system of equations using gaussian elimination.
        bool verbose;
	int i, m, n;
        long double coefficientnn;
        long double scale;

	verbose=false;

	if (verbose)
	{
		cout <<"nequations= "<<nequations<<endl;
		for (n=0;n<nequations;n++)
		{
			for (m=0;m<=nequations;m++)
			{
				cout <<coefficient[m][n]<<"\t";
			}
			cout <<endl;
		}
		cout <<endl;
	}

	for (n=0;n<nequations-1;n++)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
                        for (m=n+1;m<nequations;m++)
                        {
                                scale=coefficient[n][m]/coefficientnn;
                                for (i=0;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }
                        }
                }
        }
	
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}

        for (n=nequations-1;n>0;n=n-1)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
			for (m=n-1;m>-1;m=m-1)
                        {
                                scale=coefficient[n][m]/coefficientnn;
				//cout <<"coefficient["<<n<<"]["<<m<<"]= "<<coefficient[n][m]<<" coefficientnn= "<<coefficientnn<<" scale= "<<scale<<endl;
                                for (i=n;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }

				if (verbose)
				{
        				for (int n2=0;n2<nequations;n2++)
        				{
                				for (int m2=0;m2<=nequations;m2++)
                				{
                        				cout <<coefficient[m2][n2]<<"\t";
                				}
                				cout <<endl;
        				}
					cout <<endl;
				}
                        }
                }
        }
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}

        for (n=0;n<nequations;n++)
        {
                variable[n]=coefficient[nequations][n]/coefficient[n][n];
		if (verbose) cout <<"Variable "<<n+1<<" is "<<variable[n]<<"\n";
        }
}

void equation2(long double coefficient[][500], long double variable[], int nequations)
{
	//Finds the solution to a system of equations using gaussian elimination.
        int i, m, n;
        long double coefficientnn;
        long double scale;

	//cout <<"nequations= "<<nequations<<endl;
	//for (n=0;n<nequations;n++)
	//{
	//	for (m=0;m<=nequations;m++)
	//	{
	//		cout <<coefficient[m][n]<<"\t";
	//	}
	//	cout <<endl;
	//}
	//cout <<endl;

	for (n=0;n<nequations-1;n++)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
                        for (m=n+1;m<nequations;m++)
                        {
                                scale=coefficient[n][m]/coefficientnn;
                                for (i=0;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }
                        }
                }
        }
        for (n=0;n<nequations;n++)
        {
                for (m=0;m<=nequations;m++)
                {
                        cout <<coefficient[m][n]<<"\t";
                }
                cout <<endl;
        }
	cout <<endl;
        for (n=nequations;n>0;n=n-1)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
			for (m=n-1;m>-1;m=m-1)
                        {
                                scale=coefficient[n][m]/coefficientnn;
                                for (i=n;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }
                        }
                }
        }
        for (n=0;n<nequations;n++)
        {
                for (m=0;m<=nequations;m++)
                {
                        cout <<coefficient[m][n]<<"\t";
                }
                cout <<endl;
        }
	cout <<endl;
        for (n=0;n<nequations;n++)
        {
                variable[n]=coefficient[nequations][n]/coefficient[n][n];
                cout <<"Variable "<<n+1<<" is "<<variable[n]<<"\n";
        }
}
*/

template <typename T>
void swapEquations(vector< vector<T> > &coefficient, int i, int j)
{
	int Size=coefficient.size();

	for (int k=0;k<Size;k++)
	{
		swap(coefficient[k][i], coefficient[k][j]);
	}
}

template <typename T>
void makeDiagonalNonzero(vector< vector<T> > &coefficient, int n)
{
	int nequations=coefficient[0].size();

	for (int i=n+1;i<nequations;i++)
	{
		if (coefficient[i][n]!=0)
		{
			swapEquations(coefficient, i, n);
			break;
		}
	}
}

void equation(Matrix &coefficient, Vector &variable)
{
	//Finds the solution to a system of equations using gaussian elimination.
        bool verbose;
	int i, m, n;
	int Size1, Size2;
        Real coefficientnn;
        Real scale;
        int nequations=variable.size();

	verbose=false;

	Get2DVectorSize(coefficient, Size1, Size2);
	if (Size1!=Size2+1)
	{
		error("ERROR: In equation the matrix is the wrong size", __LINE__, __FILE__);
	}

	if (verbose)
	{
		cout <<"nequations= "<<nequations<<endl;
		for (n=0;n<nequations;n++)
		{
			for (m=0;m<=nequations;m++)
			{
				cout <<coefficient[m][n]<<"\t";
			}
			cout <<endl;
		}
		cout <<endl;
	}

	for (n=0;n<nequations-1;n++)
        {
		if (coefficient[n][n]==0) makeDiagonalNonzero(coefficient, n);
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
                        for (m=n+1;m<nequations;m++)
                        {
                                scale=coefficient[n][m]/coefficientnn;
                                for (i=0;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }
                        }
                }
        }
	
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}

        for (n=nequations-1;n>0;n=n-1)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
			for (m=n-1;m>-1;m=m-1)
                        {
                                scale=coefficient[n][m]/coefficientnn;
				//cout <<"coefficient["<<n<<"]["<<m<<"]= "<<coefficient[n][m]<<" coefficientnn= "<<coefficientnn<<" scale= "<<scale<<endl;
                                for (i=n;i<nequations+1;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
                                }

				if (verbose)
				{
        				for (int n2=0;n2<nequations;n2++)
        				{
                				for (int m2=0;m2<=nequations;m2++)
                				{
                        				cout <<coefficient[m2][n2]<<"\t";
                				}
                				cout <<endl;
        				}
					cout <<endl;
				}
                        }
                }
        }
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}
	if (verbose) cout <<"Almost at end"<<endl;
        for (n=0;n<nequations;n++)
        {
                variable[n]=coefficient[nequations][n]/coefficient[n][n];
		if (verbose) cout <<"Variable "<<n+1<<" is "<<variable[n]<<endl;
        }
}

void gaussJordanElimination(long double **coefficient, long double **InverseMatrix, int nequations)
{
	//Finds the inverse matrix using Gauss-Jordan Elimination
        bool verbose;
	int i, m, n;
        long double coefficientnn;
        long double scale;

	verbose=false;

	if (verbose)
	{
		cout <<"nequations= "<<nequations<<endl;
		for (n=0;n<nequations;n++)
		{
			for (m=0;m<=nequations;m++)
			{
				cout <<coefficient[m][n]<<"\t";
			}
			cout <<endl;
		}
		cout <<endl;
	}

	for (m=0;m<nequations;m++)
	{
		for (n=0;n<nequations;n++)
		{
			if (m==n) InverseMatrix[m][n]=1.0;
			else InverseMatrix[m][n]=0.0;
		}
	}

	for (n=0;n<nequations-1;n++)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
                        for (m=n+1;m<nequations;m++)
                        {
                                scale=coefficient[n][m]/coefficientnn;
                                for (i=0;i<nequations;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
					InverseMatrix[i][m]=InverseMatrix[i][m]-InverseMatrix[i][n]*scale;
                                }
                        }
                }
        }
	
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}

        for (n=nequations-1;n>0;n=n-1)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
			for (m=n-1;m>-1;m=m-1)
                        {
                                scale=coefficient[n][m]/coefficientnn;
				//cout <<"coefficient["<<n<<"]["<<m<<"]= "<<coefficient[n][m]<<" coefficientnn= "<<coefficientnn<<" scale= "<<scale<<endl;
                                for (i=n;i<nequations;i++)
                                {
                                        coefficient[i][m]=coefficient[i][m]-coefficient[i][n]*scale;
					InverseMatrix[i][m]=InverseMatrix[i][m]-InverseMatrix[i][n]*scale;
                                }

				if (verbose)
				{
        				for (int n2=0;n2<nequations;n2++)
        				{
                				for (int m2=0;m2<=nequations;m2++)
                				{
                        				cout <<coefficient[m2][n2]<<"\t";
                				}
                				cout <<endl;
        				}
					cout <<endl;
				}
                        }
                }
        }
	if (verbose)
	{
        	for (n=0;n<nequations;n++)
        	{
                	for (m=0;m<=nequations;m++)
                	{
                        	cout <<coefficient[m][n]<<"\t";
                	}
                	cout <<endl;
        	}
		cout <<endl;
	}
}

void gaussJordanElimination(Matrix coefficient, Matrix &InverseMatrix)
{
	//Finds the inverse matrix using Gauss-Jordan Elimination
        bool verbose;
	int i, m, n;
	int nequations=coefficient.size();
	int ninverse=InverseMatrix.size();
        long double coefficientnn;
        long double scale;

	verbose=false;

	if (verbose)
	{
		cout <<"nequations= "<<nequations<<endl;
		printMatrix(coefficient);
	}
	
	if (nequations!=ninverse) 
	{
		Safe2DAlloc(InverseMatrix, nequations, nequations, "InverseMatrix");
	}

	for (m=0;m<nequations;m++)
	{
		for (n=0;n<nequations;n++)
		{
			if (m==n) InverseMatrix[m][n]=1.0;
			else InverseMatrix[m][n]=0.0;
		}
	}
	
	for (n=0;n<nequations-1;n++)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
                        for (m=n+1;m<nequations;m++)
                        {
                                scale=coefficient[m][n]/coefficientnn;
                                for (i=0;i<nequations;i++)
                                {
                                        coefficient[m][i]-=coefficient[n][i]*scale;
					InverseMatrix[m][i]-=InverseMatrix[n][i]*scale;
                                }
                        }
                }
        }
	
	if (verbose) printMatrix(coefficient);

        for (n=nequations-1;n>0;n=n-1)
        {
                coefficientnn=coefficient[n][n];
                if (coefficientnn!=0)
                {
			for (m=n-1;m>-1;m=m-1)
                        {
                                scale=coefficient[m][n]/coefficientnn;
				//cout <<"coefficient["<<n<<"]["<<m<<"]= "<<coefficient[n][m]<<" coefficientnn= "<<coefficientnn<<" scale= "<<scale<<endl;
                                for (i=0;i<nequations;i++)
                                {
                                        coefficient[m][i]-=coefficient[n][i]*scale;
					InverseMatrix[m][i]-=InverseMatrix[n][i]*scale;
                                }

				if (verbose) printMatrix(coefficient);
                        }
                }
        }
	if (verbose) printMatrix(coefficient);
	for (n=0;n<nequations;n++)
	{
		for (m=0;m<nequations;m++) InverseMatrix[n][m]/=coefficient[n][n];
	}
}

void calcInverseMatrix(Matrix coefficient, Matrix &InverseMatrix)
{
	gaussJordanElimination(coefficient, InverseMatrix);
}

Real getMatrixElement(Real **Matrix1, Real **Matrix2, int m, int n, int Size)
{
	int i;
	Real sum=0;
	for (i=0;i<Size;i++) sum+=Matrix1[m][i]*Matrix2[i][n];
	return sum;
}

void matrixMultiply(Real **Matrix1, Real **Matrix2, Real **MatrixOut, int Size1, int Size2)
{
	int m, n;

	for (m=0;m<Size1;m++)
	{
		for (n=0;n<Size2;n++)
		{
			MatrixOut[m][n]=getMatrixElement(Matrix1, Matrix2, m, n, Size1);
		}
	}
}

Real getMatrixElement(Real Matrix1[3][3], Real Matrix2[3][3], int m, int n, int Size)
{
	int i;
	Real sum=0;
	for (i=0;i<Size;i++) sum+=Matrix1[m][i]*Matrix2[i][n];
	return sum;
}

void matrixMultiply(Real Matrix1[3][3], Real Matrix2[3][3], Real MatrixOut[3][3])
{
	int Size1, Size2;
	Size1=3;
	Size2=3;
	for (int m=0;m<Size1;m++)
	{
		for (int n=0;n<Size2;n++)
		{
			MatrixOut[m][n]=getMatrixElement(Matrix1, Matrix2, m, n, Size1);
		}
	}
}

Real getMatrixElement(Matrix Matrix1, Matrix Matrix2, int m, int n)
{
	int Size=Matrix2.size();
	Real sum=0;
	for (int i=0;i<Size;i++) sum+=Matrix1[m][i]*Matrix2[i][n];
	
	return sum;
}

Real getMatrixElement(vector<Real> &v, Matrix &mat, int n)
{
	int Size=mat.size();
	Real sum=0;
	for (int i=0;i<Size;i++) sum+=v[i]*mat[i][n];
	
	return sum;
}

void matrixMultiply(Matrix Matrix1, Matrix Matrix2, Matrix &MatrixOut)
{
	int SizeOut1, SizeOut2;
	int SizeM11, SizeM12, SizeM21, SizeM22;

	Get2DVectorSize(Matrix1, SizeM11, SizeM12, "Matrix1");
	Get2DVectorSize(Matrix2, SizeM21, SizeM22, "Matrix2");
	Get2DVectorSize(MatrixOut, SizeOut1, SizeOut2);

	if (SizeM11!=SizeM22)
	{
		string errorStr=" Matrixes cannot be multiplied because they ";
		errorStr+="are the wrong size SizeM11= "+IntToStr(SizeM11);
		errorStr+="SizeM12= "+IntToStr(SizeM12)+" SizeM21= ";
		errorStr+=IntToStr(SizeM21)+" SizeM22= "+IntToStr(SizeM22);
		error(errorStr, __LINE__, __FILE__);
	}

	if (SizeOut1!=SizeM12 || SizeOut2!=SizeM11) 
	{
		Safe2DAlloc(MatrixOut, SizeM12, SizeM11, "MatrixOut");
	}

	for (int m=0;m<SizeM11;m++)
	{
		for (int n=0;n<SizeM12;n++)
		{
			MatrixOut[m][n]=getMatrixElement(Matrix1, Matrix2, m, n);
		}
	}
}

void matrixMultiplyFast(Vector &v, Matrix &mat, Vector &vOut)
{
        vOut[0]=v[0]*mat[0][0];
        vOut[1]=v[0]*mat[0][1];
        vOut[2]=v[0]*mat[0][2];

        vOut[0]+=v[1]*mat[1][0];
        vOut[1]+=v[1]*mat[1][1];
        vOut[2]+=v[1]*mat[1][2];

        vOut[0]+=v[2]*mat[2][0];
        vOut[1]+=v[2]*mat[2][1];
        vOut[2]+=v[2]*mat[2][2];

}

void matrixMultiply(vector<Real> &v, Matrix &mat, vector<Real> &vOut)
{
	int vSize, outSize, mSize1, mSize2;

	vSize=v.size();
	outSize=vOut.size();
	Get2DVectorSize(mat, mSize1, mSize2);

	if (vSize!=outSize) SafeAlloc(vOut, vSize, "vOut");
	if (vSize!=mSize1)
	{
		cout <<"ERROR: Vector size "<<vSize<<" does not match matrix "
		<<"size "<<mSize1<<endl;
		exit(EXIT_FAILURE);
	}

	for (int i=0;i<vSize;i++)
	{
		vOut[i]=getMatrixElement(v, mat, i);
	}
}

void matrixMultiply(Matrix &mat, vector<Real> &v, vector<Real> &vOut)
{
	int vSize, mSize1, mSize2;

	vSize=v.size();
	Get2DVectorSize(mat, mSize1, mSize2);

	SafeAlloc(vOut, vSize, "vOut");
	if (vSize!=mSize1)
	{
		cout <<"ERROR: Vector size "<<vSize<<" does not match matrix "
		<<"size "<<mSize1<<endl;
		exit(EXIT_FAILURE);
	}

	for (int i=0;i<vSize;i++)
	{
		vOut[i]=calcDotProduct(v, mat[i]);
	}
}

void matrixMultiply(Real x1, Real y1, Real z1, Matrix &mat, Real &x2, Real &y2, Real &z2)
{
	int Size1, Size2;

	Get2DVectorSize(mat, Size1, Size2);

	if (Size1!=3 || Size2!=3)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	x2=mat[0][0]*x1+mat[0][1]*y1+mat[0][2]*z1;
	y2=mat[1][0]*x1+mat[1][1]*y1+mat[1][2]*z1;
	z2=mat[2][0]*x1+mat[2][1]*y1+mat[2][2]*z1;
}

void subtractMatrix(Matrix &mat1, Matrix &mat2, Matrix &diffMat)
{
	int Size1a, Size2a;
	int Size1b, Size2b;
	int Size1c, Size2c;

	Get2DVectorSize(mat1, Size1a, Size2a, "mat1");
	Get2DVectorSize(mat2, Size1b, Size2b, "mat2");
	Get2DVectorSize(diffMat, Size1c, Size2c);

	if (Size1a!=Size1b || Size2a!=Size2b)
	{
		string errorStr="Size1a= "+toStr(Size1a);
		errorStr+="Size1b= "+toStr(Size1b);
		errorStr+="Size2a= "+toStr(Size2a);
		errorStr+="Size2b= "+toStr(Size2b);
		error(errorStr, __LINE__, __FILE__);
	}
	if (Size1c!=Size1a || Size2c!=Size2a)
	{
		Safe2DAlloc(diffMat, Size1a, Size2a, "diffMat");
	}

	for (int i=0;i<Size1a;i++)
	{
		for (int j=0;j<Size2a;j++)
		{
			diffMat[i][j]=mat1[i][j]-mat2[i][j];
		}
	}
}

Real calcTrace(Matrix &mat)
{
	int Size1, Size2;
	Real trace=0;
	Get2DVectorSize(mat, Size1, Size2, "mat");

	if (Size1!=Size2)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

	for (int i=0;i<Size1;i++)
	{
		trace+=mat[i][i];
	}
	return trace;
}

void transpose(Matrix Mat, Matrix &Matrixtranspose)
{
	int Size1=0, Size2=0;
	int TSize1=0, TSize2=0;
	Size1=Mat.size();
	if (Size1>0 )Size2=Mat[0].size();
	TSize1=Matrixtranspose.size();
	if (TSize1>0)  TSize2=Matrixtranspose[0].size();

	if (TSize1!=Size2 || TSize2!=Size1)
	{
		deleteMatrix(Matrixtranspose);
		CreateMatrix(Matrixtranspose, Size2, Size1);
	}

	for (int i=0;i<Size1;i++)
	{
		for (int j=0;j<Size2;j++) Matrixtranspose[j][i]=Mat[i][j];
	}
}

template<class Matrix_T>
void transpose(Matrix_T &Matrix)
{
	int i, j;
	int Size1, Size2;
	Matrix_T Matrixtranspose;

	Size1=Matrix.size();
	Size2=Matrix[0].size();

	CreateMatrix(Matrixtranspose, Size2, Size1);

	for (i=0;i<Size1;i++)
	{
		for (j=0;j<Size2;j++) Matrixtranspose[j][i]=Matrix[i][j];
	}
	CopyVector(Matrixtranspose, Matrix);
}

void swap(Real &x, Real &y)
{
	Real temp;

	temp=x;
	x=y;
	y=temp;
}

template <class Matrix_T>
void transpose(Matrix_T &Matrix, int Size1, int Size2)
{
	//Only works for square matrix.

	for (int i=0;i<Size1;i++)
	{
		for (int j=i;j<Size2;j++) 
		{
			swap(Matrix[i][j], Matrix[j][i]);
		}
	}
}

template <class Matrix_T>
void transpose2(Matrix_T &Matrix, int Size1, int Size2)
{
	//Only works for square matrix.

	for (int i=0;i<Size1;i++)
	{
		for (int j=i;j<Size2;j++) 
		{
			for (int k=0;k<Size1;k++)
			{
				swap(Matrix[i][k][j], Matrix[j][k][i]);
			}
		}
	}
}

void normalizeMatrix(Matrix &mat)
{
	int Size1, Size2;
	Real sum=0;

	Get2DVectorSize(mat, Size1, Size2, "mat");

	for (int i=0;i<Size1;i++)
	{
		for (int j=0;j<Size2;j++)
		{
			sum+=mat[i][j]*mat[i][j];
		}
	}
	sum=sqrt(sum);
	for (int i=0;i<Size1;i++)
	{
		for (int j=0;j<Size2;j++)
		{
			mat[i][j]/=sum;
		}
	}
}

void calcEigenvector(Matrix &mat, Vector &eigenvector)
{
	int Size=mat.size();
	Real diff, criterion=1e-10;
	Vector eigenvector2;
	Matrix mat2, mat3;

	SafeAlloc(eigenvector, Real(1.0), Size, "eigenvector");
	normalize(eigenvector);
	mat2=mat;
	while (true)
	{
		matrixMultiply(mat, mat2, mat3);
		normalizeMatrix(mat3);
		mat2=mat3;
		matrixMultiply(mat3, eigenvector, eigenvector2);
		normalize(eigenvector2);
		diff=calcDistance(eigenvector, eigenvector2);
		if (diff<criterion) break;
		eigenvector=eigenvector2;
	}
}

Real calcSum(vector<Real> &x)
{
        int Size=x.size();
        Real sum=0;

        for (int i=0;i<Size;i++) sum+=x[i];

        return sum;
}

Real calcSumSqr(vector<Real> &x)
{
        int Size=x.size();
        Real sum=0;

        for (int i=0;i<Size;i++) sum+=x[i]*x[i];

        return sum;
}
/*
template<class T>
T calcAverage(vector<T> &x)
{
	int i, Size=x.size();
	Real sum=0;
        if (Size==0) return 0;
	for (i=0;i<Size;i++) sum+=x[i];

	return sum/Real(Size);
}

template<class T>
T calcAverage(vector<T> &x, int first, int last)
{
	int i, Size=x.size();
	Real sum=0;
        if (Size==0) return 0;
	for (i=first;i<=last;i++) sum+=x[i];

	return sum/Real(last-first+1);
}
*/
Real calcAverage(vector<Real> &x)
{
	int i, Size=x.size();
	Real sum=0;
        if (Size==0) return 0;
	for (i=0;i<Size;i++) sum+=x[i];

	return sum/Real(Size);
}

Real calcAverage(vector<Real> &x, int first, int last)
{
	int i, Size=x.size();
	Real sum=0;
        if (Size==0) return 0;
	for (i=first;i<=last;i++) sum+=x[i];

	return sum/Real(last-first+1);
}

Real calcVariance(vector<Real> &x)
{
        int Size=x.size();
        Real sum, sum_sqr;
        sum=0;
        sum_sqr=0;
        for (int i=0;i<Size;i++)
        {
                sum+=x[i];
                sum_sqr+=x[i]*x[i];
        }
        return (sum_sqr/Real(Size)-sum*sum/(Real(Size)*Real(Size)));
}

Real calcStandardDeviation(vector<Real> &x)
{
        return sqrt(calcVariance(x));
}

Real calcError(Vector &x)
{
        return calcStandardDeviation(x)/sqrt(Real(x.size()));
}

Real variance(vector< vector<Real> > &x)
{
	int i, j, Size1, Size2;
	Real sum, sum_sqr, NumElements;
	Size1=x.size();
	Size2=x[0].size();
	sum=0;
	sum_sqr=0;
	for (i=0;i<Size1;i++)
	{
		for (j=0;j<Size2;j++)
		{
			sum+=x[i][j];
			sum_sqr+=x[i][j]*x[i][j];
		}
	}

	NumElements=Real(Size1*Size2);
	return (sum_sqr/NumElements-sum*sum/(NumElements*NumElements));
}

template <class T>
Real calcCovariance(vector<T> &x, vector<T> &y)
{
	int Size=x.size();
	Real sum=0;
	Real AverageX, AverageY;

	AverageX=calcAverage(x);
	AverageY=calcAverage(y);

	for (int n=0;n<Size;n++) sum+=(x[n]-AverageX)*(y[n]-AverageY);

	return sum;
}

template<class Matrix_T>
void calcCovarianceMatrix(Matrix_T Matrix, Matrix_T &CovarianceMatrix)
{
	int i, j;
	int Size1, Size2;

	Size1=Matrix.size();
	Size2=Matrix[0].size();

	if (CovarianceMatrix.size()!=Size1 || CovarianceMatrix[0].size()!=Size1)
	{
		deleteMatrix(CovarianceMatrix);
		CreateMatrix(CovarianceMatrix, Size1, Size1);
	}

	for (i=0;i<Size1;i++)
	{
		for (j=i;j<Size1;j++)
		{
			CovarianceMatrix[i][j]=calcCovariance(Matrix[j], Matrix[i]);
			CovarianceMatrix[j][i]=CovarianceMatrix[i][j];
		}
	}
}

void calcZScore(vector<Real> &x, vector<Real> &z)
{
	int Size=x.size();
	Real ave, stdDev;
	
	SafeAlloc(z, Size, "z");
	ave=calcAverage(x);
	stdDev=calcStandardDeviation(x);
	for (int i=0;i<Size;i++) z[i]=(x[i]-ave)/stdDev;
}

Real interpolate(Real x1, Real y1, Real x2, Real y2, Real x)
{
	//Given data points (x1, y1) and (x2, y2) performs linear interpolation
	//to give the value of y at x

	Real dy, dx, slope, y;

	if (x1==x2) return (y1+y2)*0.5;
	
	dy=y2-y1;
	dx=x2-x1;
	slope=dy/dx;
	y=y1+(x-x1)*slope;
/*	
	if (isnan(y))
	{
		cout <<"("<<x1<<", "<<y1<<") ("<<x2<<", "<<y2<<")"<<endl;
		exit(EXIT_FAILURE);
	}
*/
	return y;
}

Real interpolate3D(vector< vector< vector<PosStruct> > > &x, Array3D &y, vector<Real> &pos)
{
	//en.wikipedia.org/wiki/Trilinear_interpolation

	Real c00, c10, c01, c11;
	Real c0, c1, c;

	c00=interpolate(x[0][0][0].Pos[X], y[0][0][0], x[1][0][0].Pos[X], y[1][0][0], pos[X]);
	c10=interpolate(x[0][0][0].Pos[X], y[0][1][0], x[1][0][0].Pos[X], y[1][1][0], pos[X]);
	c01=interpolate(x[0][0][0].Pos[X], y[0][0][1], x[1][0][0].Pos[X], y[1][0][1], pos[X]);
	c11=interpolate(x[0][0][0].Pos[X], y[0][1][1], x[1][0][0].Pos[X], y[1][1][1], pos[X]);

	c0=interpolate(x[0][0][0].Pos[Y], c00, x[0][1][0].Pos[Y], c10, pos[Y]);
	c1=interpolate(x[0][0][0].Pos[Y], c01, x[0][1][0].Pos[Y], c11, pos[Y]);

	c=interpolate(x[0][0][0].Pos[Z], c0, x[0][0][1].Pos[Z], c1, pos[Z]);

	return c;
} 

Real calcAreaUnderCurve(vector<Real> &x, vector<Real> &y)
{
	int Size1=x.size();
	int Size2=y.size();
	Real sum=0;

	if (Size1==0)
	{
		cout <<"ERROR: Vector size is 0 in calcAreaUnderCurve"<<endl;
		exit(EXIT_FAILURE);
	}

	if (Size1!=Size2)
	{
		cout <<"ERROR: x.size()= "<<Size1<<" y.size()= "<<Size2<<endl;
		exit(EXIT_FAILURE);
	}

	for (int i=0;i<Size1-1;i++)
	{
		sum+=(y[i]+y[i+1])*(x[i+1]-x[i])*0.5;
	}
	return sum;
}

void calcBinAverage(Real binSize, vector<Real> &x, vector<Real> &y, vector<Real> &ave)
{
	int Size1=x.size();
	int Size2=y.size();
	int maxBin, bin;
	Real min, max;
	vector<Real> total, num;

	if (Size1!=Size2)
	{
		cout <<"ERROR: Size1= "<<Size1<<" Size2= "<<Size2
		<<" in calcBinAverage"<<endl;
		exit(EXIT_FAILURE);
	}
	
	MinMax(x, min, max);
	maxBin=int((max-min)/binSize+0.5)+1;
	SafeAlloc(total, maxBin);
	SafeAlloc(num, maxBin);
	SafeAlloc(ave, maxBin);

	for (int i=0;i<Size1;i++)
	{
		bin=int((x[i]-min)/binSize+0.5);
		total[bin]+=y[i];
		num[bin]+=1.0;
	}

	for (int i=0;i<maxBin;i++)
	{
		ave[i]=total[i]/num[i];
	}
}

bool isNonzero(Vector &v)
{
	int Size=v.size();

	for (int i=0;i<Size;i++)
	{
		if (v[i]!=0) return true;
	}
	return false;
}

void normalize(vector<Real> &v)
{
        int Size=v.size();
        Real sum=calcSum(v);

	if (sum==0) error("Vector cannot be normalized", __LINE__, __FILE__);
        for (int i=0;i<Size;i++) v[i]/=sum;
}

void setAverageTo(vector<Real> &v, Real x)
{
        int Size=v.size();
        Real sum=calcSum(v);
        for (int i=0;i<Size;i++) v[i]=v[i]*x/sum;
}

Real getMin(Real x1, Real x2)
{
        if (x1<=x2) return x1;
        else return x2;
}

Real getMax(Real x1, Real x2)
{
        if (x1<=x2) return x2;
        else return x1;
}

int getMin(int x1, int x2)
{
        if (x1<=x2) return x1;
        else return x2;
}

int getMax(int x1, int x2)
{
        if (x1<=x2) return x2;
        else return x1;
}

Real calcMonotonicity(Vector &x, Vector &y)
{
	//Calculates the probability that if y decreases x decreases.
	int Size, inc=1;
	Real monatonicity, numLess, xm, ym;
	timeval start, end;
	gettimeofday(&start, NULL);
	Size=x.size();
	monatonicity=0;
	numLess=0;
	if (Size>10000) inc=100;
	for (int m=0;m<Size-1;m+=inc)
	{
		xm=x[m]; ym=y[m];
		for (int n=m+1;n<Size;n+=inc)
		{
			if (ym<y[n]) 
			{
				numLess+=1.0;
				if (xm<x[n]) monatonicity+=1.0;
			}
			if (ym>y[n]) 
			{
				numLess+=1.0;
				if (xm>x[n]) monatonicity+=1.0;
			}
		}
	}
	gettimeofday(&end, NULL);
	//cout <<"Time in Monotonicity= "<<calcTimeDiff(start, end)<<endl;
	monatonicity=monatonicity/numLess;
	return monatonicity;
}
/*
template <class Vector_T>
bool Issorted(Vector_T &v)
{
	int n;
	int Size=v.size();
	if (Size>1)
	{
		for (n=1;n<Size;n++) 
		{
			if (v[n]<v[n-1]) return false;
		}
	}

	return true;

}

template <class Vector_T>
void sort(Vector_T &v)
{
	bool sorted1, sorted2;
	int n;
	int Size, Size1, Size2;
	Real first, last;
	vector<Real> v1, v2;

	Size=v.size();
	first=v[0];
	for (n=0;n<Size;n++)
	{
		if (v[n]<first)
		{
			first=v[n];
			break;
		}
		else if (v[n]>first) break;
	}

	for (n=0;n<Size;n++)
	{
		if (v[n]<=first) v1.push_back(v[n]);
		else v2.push_back(v[n]);
	}

	Size1=v1.size();
	Size2=v2.size();
	//for (n=0;n<Size1;n++) cout <<"v1["<<n<<"]= "<<v1[n]<<endl;
	//cout <<endl;
	//for (n=0;n<Size2;n++) cout <<"v2["<<n<<"]= "<<v2[n]<<endl;
	//cout <<endl;
	sorted1=isSorted(v1);
	sorted2=isSorted(v2);
	if (Size1>1 && !sorted1) sort(v1);
	if (Size2>1 && !sorted2) sort(v2);

	for (n=0;n<Size;n++) v.pop_back();

	for (n=0;n<Size1;n++)
	{
		v.push_back(v1[n]);
	}

	for (n=0;n<Size2;n++)
	{
		v.push_back(v2[n]);
	}

}
*/

bool isSorted(Vector &v)
{
	int n;
	int Size=v.size();
	if (Size>1)
	{
		for (n=1;n<Size;n++) 
		{
			if (v[n]<v[n-1]) return false;
		}
	}

	return true;
}

bool isSorted(vector<int> &v)
{
	int n;
	int Size=v.size();
	if (Size>1)
	{
		for (n=1;n<Size;n++) 
		{
			if (v[n]<v[n-1]) return false;
		}
	}

	return true;
}

void sort(Vector &v)
{
	bool sorted1, sorted2;
	int Size, Size1, Size2;
	Real first;
	Vector v1, v2;


	Size=v.size();

	if (Size==0)
	{
		error("Size= 0", __LINE__, __FILE__);
	}
	first=v[0];
	for (int n=0;n<Size;n++)
	{
		if (v[n]<first)
		{
			first=v[n];
			break;
		}
		else if (v[n]>first) break;
	}

	for (int n=0;n<Size;n++)
	{
		if (v[n]<=first) SafePushBack(v1, v[n], "v1");
		else SafePushBack(v2, v[n], "v2");
	}

	Size1=v1.size();
	Size2=v2.size();
	//for (n=0;n<Size1;n++) cout <<"v1["<<n<<"]= "<<v1[n]<<endl;
	//cout <<endl;
	//for (n=0;n<Size2;n++) cout <<"v2["<<n<<"]= "<<v2[n]<<endl;
	//cout <<endl;
	sorted1=isSorted(v1);
	sorted2=isSorted(v2);
	if (Size1>1 && !sorted1) sort(v1);
	if (Size2>1 && !sorted2) sort(v2);

        v.clear();

	for (int n=0;n<Size1;n++)
	{
                SafePushBack(v, v1[n], "v");
	}

	for (int n=0;n<Size2;n++)
	{
                SafePushBack(v, v2[n], "v");
	}
}

void sort(vector<int> &v)
{
	bool sorted1, sorted2;
	int n;
	int Size, Size1, Size2;
	Real first;
	vector<int> v1, v2;

	Size=v.size();
	first=v[0];
	for (n=0;n<Size;n++)
	{
		if (v[n]<first)
		{
			first=v[n];
			break;
		}
		else if (v[n]>first) break;
	}

	for (n=0;n<Size;n++)
	{
		if (v[n]<=first) SafePushBack(v1, v[n], "v1");
		else SafePushBack(v2, v[n], "v2");
	}

	Size1=v1.size();
	Size2=v2.size();
	sorted1=isSorted(v1);
	sorted2=isSorted(v2);
	if (Size1>1 && !sorted1) sort(v1);
	if (Size2>1 && !sorted2) sort(v2);

        v.clear();

	for (n=0;n<Size1;n++)
	{
                SafePushBack(v, v1[n], "v");
	}

	for (n=0;n<Size2;n++)
	{
                SafePushBack(v, v2[n], "v");
	}
}

Real calcGaussian(Real x, Real stdDev)
{
        return exp(-x*x/(2.0*stdDev*stdDev))/(stdDev*sqrt(2.0*pi));
}

Real calcGaussian(Real x, Real mean, Real stdDev)
{
        return calcGaussian(x-mean, stdDev);
}

void initializeRandom(int rannum)
{
	srand(rannum);
}

int randInt(int Max)
{
	int Rand;
	Rand=int(Real(rand())*Real(Max)/Real(RAND_MAX)+0.5);
	return Rand;
}

int randInt(int Min, int Max)
{
        return randInt(Max-Min)+Min;
}

Real randDouble(Real Max)
{
	Real Rand;
	Rand=2.0*Max*Real(rand())/Real(RAND_MAX)-Max;
	return Rand;
}

Real randDouble(Real Min, Real Max)
{
        return (Max-Min)*Real(rand())/Real(RAND_MAX)+Min;
}

Real randDoubleRef(Real &Min, Real &Max)
{
        return (Max-Min)*Real(rand())/Real(RAND_MAX)+Min;
}

Real randGaussian(Real stdDev)
{
        if (isnan(stdDev) || isinf(stdDev)) return randDouble(-1.0, 1.0);
	if (stdDev==0) return 0;
        Real max=4.0*stdDev;
        Real gaussian, randNum, randNum2;
        randNum=randDouble(-max, max);
        //gaussian=calcGaussian(randNum, stdDev);
        gaussian=exp(-randNum*randNum/(2.0*stdDev*stdDev));
        randNum2=randDouble(0, 1.0);
        if (gaussian>randNum2) return randNum;
        else return randGaussian(stdDev);
}

Real randGaussian(Real mean, Real stdDev)
{
        return randGaussian(stdDev)+mean;
}

Real randAngle(Real mean, Real width)
{
        Real sign;
        Real angle=randGaussian(mean, width);
        angle=calcDeltaAngle(0, angle);
        if (angle>pi) angle=2.0*pi-angle;
        else if (angle<-pi) angle=2.0*pi+angle;
        sign=randDouble(-1.0, 1.0);
        if (sign<0) return -angle;
        return angle;
}

VectorStruct randVector(Real length)
{
	Real r;
	VectorStruct v;
	v.x=randDouble(-1.0, 1.0);
	v.y=randDouble(-1.0, 1.0);
	v.z=randDouble(-1.0, 1.0);
	r=vectorLength(v);
	if (r>1.0) return randVector(length);
	v=v/r;
	v=v*length;

	return v;
}

VectorStruct randVector()
{
	return randVector(1.0);
}

void createRandomVector(Vector &r, int Size)
{
	SafeAlloc(r, Size, "r");
	for (int i=0;i<Size;i++)
	{
		r[i]=randDouble(0, 1.0);
	}
	setLength(r, 1.0);
}

/*
template <typename T>
void linearRegression(vector<T> x, vector<T> y, T &slope, T &yintercept)
{
	int Size;
	vector<long double> variable;
	vector< vector<long double> > coefficient;
	//cout <<"In FitLine"<<endl;
	CreateVector(variable, 2);
	CreateMatrix(coefficient, 3, 2);
	//cout <<"Initialized coefficeint"<<endl;
	Size=x.size();
	coefficient[0][0]=(long double)(Size);
	for (int n=0;n<Size;n++) coefficient[1][0]+=x[n];
	for (int n=0;n<Size;n++) coefficient[2][0]+=y[n];
	coefficient[0][1]=coefficient[1][0];
	for (int n=0;n<Size;n++) coefficient[1][1]+=x[n]*x[n];
	for (int n=0;n<Size;n++) coefficient[2][1]+=x[n]*y[n];
	//cout <<"Assigned coefficients"<<endl;
	Equation(coefficient, variable);
	//cout <<"Solved equation"<<endl;
	yintercept=variable[0];
	slope=variable[1];
}
*/
void linearRegression(Vector &x, Vector &y, Real &slope, Real &yintercept)
{
	int Size;
	vector<Real> variable;
	vector< vector<Real> > coefficient;
	//cout <<"In FitLine"<<endl;
	CreateVector(variable, 2);
	CreateMatrix(coefficient, 3, 2);
	//cout <<"Initialized coefficeint"<<endl;
	Size=x.size();
	coefficient[0][0]=Real(Size);
	for (int n=0;n<Size;n++) coefficient[1][0]+=x[n];
	for (int n=0;n<Size;n++) coefficient[2][0]+=y[n];
	coefficient[0][1]=coefficient[1][0];
	for (int n=0;n<Size;n++) coefficient[1][1]+=x[n]*x[n];
	for (int n=0;n<Size;n++) coefficient[2][1]+=x[n]*y[n];
	//cout <<"Assigned coefficients"<<endl;
	equation(coefficient, variable);
	//cout <<"Solved equation"<<endl;
	yintercept=variable[0];
	slope=variable[1];
}

Real calcAbsSum(vector<Real> &v)
{
        int Size=v.size();
        Real sum=0;

        for (int i=0;i<Size;i++)
        {
                sum+=abs(v[i]);
        }
        return sum;
}

void abs(vector<Real> &v)
{
        int Size=v.size();
        for (int i=0;i<Size;i++)
        {
                v[i]=abs(v[i]);
        }
}

Real calcAverage(vector<Real> &x, vector<Real> &y)
{
        int Size=x.size();
        Real sum=0;

        for (int i=0;i<Size;i++)
        {
                sum+=x[i]*abs(y[i]);
        }
        return sum/calcAbsSum(y);
}

void calcStdDevAndAverage(vector<Real> &x, vector<Real> &y, Real &ave, Real &stdDev)
{
        int Size=x.size();
        Real sum=0, sumSqr=0, sumY=0;
        Real aveSqr, sqrAve;

        for (int i=0;i<Size;i++)
        {
                sumSqr+=x[i]*x[i]*abs(y[i]);
                sum=x[i]*abs(y[i]);
        }
        sumY=calcAbsSum(y);
        ave=sum/sumY;
        aveSqr=sumSqr/sumY;
        sqrAve=ave*ave;
        stdDev=sqrt(aveSqr-sqrAve);
}

Real calcScaleFactor(vector<Real> &y1, vector<Real> &y2)
{
        //Minimize sum((scale*y1-y2)^2)
        int Size=y1.size();
        Real sumCrossTerm=0, sumSqr=0;

        for (int i=0;i<Size;i++)
        {
                sumCrossTerm+=y1[i]*y2[i];
                sumSqr+=y1[i]*y1[i];
                cout <<"y1= "<<y1[i]<<" y2= "<<y2[i]<<endl;
                cout <<"sumCrossTerm= "<<sumCrossTerm<<" sumSqr= "<<sumSqr<<endl;
        }
        return sumCrossTerm/sumSqr;
}

void calcGaussian(vector<Real> &x, vector<Real> &gaussian, Real mean, Real width)
{
        int Size=x.size();
        cout <<"in calcGaussian vector"<<endl;
        if (gaussian.size()==0) SafeAlloc(gaussian, Size, "gaussian");
        for (int i=0;i<Size;i++)
        {
                gaussian[i]=calcGaussian(x[i], mean, width);
                //cout <<"gaussian["<<i<<"]= "<<gaussian[i]<<" x= "<<x[i]
                //<<" mean= "<<mean<<" width= "<<width<<endl;
        }
}

Real calcScaleForGaussian(vector<Real> &x, vector<Real> &y, Real mean, Real width)
{
        vector<Real> gaussian;
        calcGaussian(x, gaussian, mean, width); 
        return calcScaleFactor(gaussian, y);
}

Real calcSqrDiff(vector<Real> &y1, vector<Real> &y2)
{
        int Size=y1.size();
        Real sum=0;
        for (int i=0;i<Size;i++)
        {
                sum+=(y2[i]-y1[i])*(y2[i]-y1[i]);
        }
        return sum;
}

void vectorMultiply(Real scale, vector<Real> &v)
{
        int Size=v.size();
        for (int i=0;i<Size;i++)
        {
                v[i]*=scale;
        }
}

vector<Real> multiply(Real scale, vector<Real> &v)
{
	int Size=v.size();
	vector<Real> v2;
	SafeAlloc(v2, Size, "multiply");
	for (int i=0;i<Size;i++)
	{
		v2[i]=v[i]*scale;
	}
	return v2;
}

void multiply(Real scale, vector<Real> &v1, vector<Real> &v2)
{
	int Size1=v1.size();
	int Size2=v2.size();
	if (Size1!=Size2) SafeAlloc(v2, Size1, "multiply");
	for (int i=0;i<Size1;i++)
	{
		v2[i]=v1[i]*scale;
	}
}

vector<Real> addVectors(vector<Real> &v1, vector<Real> &v2)
{
	int Size1=v1.size();
	int Size2=v2.size();
	vector<Real> v3;

	if (Size1!=Size2)
	{
		cout <<"ERROR: Size1= "<<Size1<<" Size2= "<<Size2
		<<" in addVectors"<<endl;
		exit(EXIT_FAILURE);
	}

	SafeAlloc(v3, Size1, "v3 in addVectors");
	for (int i=0;i<Size1;i++)
	{
		v3[i]=v1[i]+v2[i];
	}
	return v3;
}

Real gaussianFitScore(vector<Real> &x, vector<Real> &y, Real mean, Real width)
{
        Real scale;
        vector<Real> gaussian;
        cout <<"in gaussianFitScore"<<endl;
        calcGaussian(x, gaussian, mean, width);
        scale=calcScaleFactor(gaussian, y);
        cout <<"scale= "<<scale<<endl;
        vectorMultiply(scale, gaussian);
        return calcSqrDiff(gaussian, y);
}

Real gaussianFitScore(vector<Real> coefficient, xyStruct xy)
{
        Real fitScore=gaussianFitScore(xy.x, xy.y, coefficient[0], coefficient[1]);
        cout <<"fitScore= "<<fitScore<<endl;
        if (isnan(fitScore)) exit(EXIT_FAILURE);
        return fitScore;
}

void fitGaussian(vector<Real> &x, vector<Real> &y, Real &scale, Real &mean, Real &width)
{
        //y=scale*exp(-(x-mean)^2/width^0.5)
        //Real dScoreCriteria=0.0001;
        //vector<Real> coefficients;
        //xyStruct xy;
        //Real (*func_to_minimize)(vector<Real>, xyStruct)=gaussianFitScore;
        //coefficients.resize(2);
        //xy.x=x;
        //xy.y=y;
        calcStdDevAndAverage(x, y, mean, width);
        //coefficients[0]=mean;
        //coefficients[1]=width;
        cout <<" mean= "<<mean<<" width= "<<width<<endl;
        //nonlinearConjugateGradient(coefficients, xy, func_to_minimize, dScoreCriteria);
        //mean=coefficients[0];
        //width=coefficients[1];
}

void fitGaussian(vector<Real> &x, vector<Real> &y, Real &mean, Real &width)
{
        Real scale;
        fitGaussian(x, y, scale, mean, width);
}

vector<Real> calcLog(vector<Real> &v)
{
        int Size=v.size();
        vector<Real> logv;
        SafeAlloc(logv, Size, "logv");
        for (int i=0;i<Size;i++)
        {
                logv[i]=log(v[i]);
        }
        return logv;
}

Real calcPower(Real x, int degree)
{
        Real power=1.0;
        for (int i=1;i<=degree;i++)
        {
                power*=x;
        }
	if (degree<0)
	{
		for (int i=-1;i>=degree;i--)
		{
			power/=x;
		}
	}
	if (degree==0) return 1.0;
        return power;
}

void calcPowers(Real x, int degree, Vector &powers)
{
        powers[0]=1.0;
        for (int i=1;i<degree;i++)
        {
                powers[i]=powers[i-1]*x;
        }
}

vector<Real> calcPower(vector<Real> &x, int degree)
{
        int Size=x.size();
        vector<Real> power;
        SafeAlloc(power, Size, "power");
        for (int i=0;i<Size;i++)
        {
                power[i]=calcPower(x[i], degree);
        }
        return power;
}

vector<Real> fitCurves(Matrix &curves, vector<Real> &y)
{
        int Size1, Size2, npoint=y.size();
        vector<Real> variable;
        Matrix coefficients;

        Get2DVectorSize(curves, Size1, Size2);
        SafeAlloc(variable, Size2, "variables in fitCurves");
        Safe2DAlloc(coefficients, Size2+1, Size2, "in fitCurves");
        for (int i=0;i<npoint;i++)
        {
                for (int j=0;j<Size2;j++)
                {
                        for (int k=j;k<Size2;k++)
                        {
                                coefficients[k][j]+=curves[i][j]*curves[i][k];
                                coefficients[j][k]=coefficients[k][j];
                        }
                        coefficients[Size2][j]+=y[i]*curves[i][j];
                }
        }
	//cout <<"coefficients: "<<endl;
	//printMatrix(coefficients);
        equation(coefficients, variable);
        return variable;
}

Real calcOverallCorrelation(Matrix &data, Vector &z, Vector &variable)
{
        int Size1, Size2;
        Real sum, r;
        Vector x, y;

        Get2DVectorSize(data, Size1, Size2, "data");

        for (int i=0;i<Size1;i++)
        {
                sum=0;
                for (int j=0;j<Size2;j++)
                {
                        sum+=data[i][j]*variable[j];
                }
                SafePushBack(y, sum, "y");
        }
        r=correlation(y, z);

	return r;
}

Real fitCurves(Matrix &curves, Vector &y, Vector &variable)
{
	variable=fitCurves(curves, y);
	return calcOverallCorrelation(curves, y, variable);
}

vector<Real> fitPolynomial(vector<Real> &x, vector<Real> &y, int degree)
{
        vector<Real> powers, coefficients;
        Matrix polynomials;
        for (int i=0;i<degree+1;i++)
        {
                powers=calcPower(x, i);
                SafePushBack(polynomials, powers, "polynomials");
        }
        transpose(polynomials);
        coefficients=fitCurves(polynomials, y);
        return coefficients;
}

void fitQuadratic(Real x1, Real y1, Real x2, Real y2, Real x3, Real y3, Real &a, Real &b, Real &c)
{
	//Fits a quadratic equation to three points by solving the following
	//system of equations.
	//y1=a*x1^2+b*x1+c
	//y2=a*x2^2+b*x2+c
	//y3=a*x3^2+b*x3+c
	Vector x, y, coefficients;
	
	SafeAlloc(x, 3, "x");
	SafeAlloc(y, 3, "y");
	x[0]=x1; x[1]=x2; x[2]=x3;
	y[0]=y1; y[1]=y2; y[2]=y3;
	coefficients=fitPolynomial(x, y, 2);

	a=coefficients[2];
	b=coefficients[1];
	c=coefficients[0];
}

void fitGaussian2(vector<Real> &x, vector<Real> &y, Real &scale, Real &mean, Real &width)
{
        int degree=2;
        vector<Real> logy, coefficients;

        logy=calcLog(y);
        coefficients=fitPolynomial(x, logy, degree);
        width=-coefficients[2];
        mean=-coefficients[1]*0.5/coefficients[2];
        scale=exp(coefficients[0]-mean*mean*width);
        width=1.0/width;
}

void fitGaussian2(vector<Real> &x, vector<Real> &y, Real &mean, Real &width)
{
        Real scale=0;
        fitGaussian2(x, y, scale, mean, width);
}


void FFT(Real InputReal[], Real InputImag[], Real OutputReal[], Real OutputImag[], int max)
{
	int n, m;
	int halfmax;
	Real DoubleHalfMax, CosReal, CosImag, SinReal, SinImag;
	Real *EvenInputReal, *EvenInputImag, *OddInputReal, *OddInputImag;
	Real *EvenOutputReal, *EvenOutputImag, *OddOutputReal, *OddOutputImag;

	DoubleHalfMax=Real(max)*0.5;
	halfmax=int(floor(DoubleHalfMax));

	EvenInputReal=new Real[halfmax];
	EvenInputImag=new Real[halfmax];
	OddInputReal=new Real[halfmax];
	OddInputImag=new Real[halfmax];

	EvenOutputReal=new Real[halfmax];
	EvenOutputImag=new Real[halfmax];
	OddOutputReal=new Real[halfmax];
	OddOutputImag=new Real[halfmax];

	for (int i=0;i<halfmax;i++)
	{
		EvenInputReal[i]=0;
		EvenInputImag[i]=0;
		OddInputReal[i]=0;
		OddInputImag[i]=0;
		EvenOutputReal[i]=0;
		EvenOutputImag[i]=0;
		OddOutputReal[i]=0;
		OddOutputImag[i]=0;
	}

	n=0;
	m=0;

	if (max>1)
	{
		while (n<max)
		{
			EvenInputReal[m]=InputReal[n];
			EvenInputImag[m]=InputImag[n];
			n++;
			OddInputReal[m]=InputReal[n];
			OddInputImag[m]=InputImag[n];
			n++;
			m++;
		}

		FFT(EvenInputReal, EvenInputImag, EvenOutputReal, EvenOutputImag, halfmax);
		FFT(OddInputReal, OddInputImag, OddOutputReal, OddOutputImag, halfmax);

		for (int n=0;n<halfmax;n++)
                {
                        CosReal=cos(2.0*pi*Real(n)/Real(max))*OddOutputReal[n];
                        SinImag=sin(2.0*pi*Real(n)/Real(max))*OddOutputImag[n];
                        CosImag=cos(2.0*pi*Real(n)/Real(max))*OddOutputImag[n];
                        SinReal=sin(2.0*pi*Real(n)/Real(max))*OddOutputReal[n];
			OutputReal[n]=EvenOutputReal[n]+CosReal+SinImag;
			OutputImag[n]=EvenOutputImag[n]+CosImag-SinReal;
		        OutputReal[n+halfmax]=EvenOutputReal[n]-CosReal-SinImag;
                        OutputImag[n+halfmax]=EvenOutputImag[n]-CosImag+SinReal;
                }
	}
	else
	{
		OutputReal[0]=InputReal[0];
		OutputImag[0]=InputImag[0];
	}

	delete [] EvenInputReal;
	delete [] EvenInputImag;
	delete [] OddInputReal;
	delete [] OddInputImag;

	delete [] EvenOutputReal;
	delete [] EvenOutputImag;
	delete [] OddOutputReal;
	delete [] OddOutputImag;
}

void printArray(Real **output, int MAX)
{
	for (int i=0;i<MAX;i++)
	{
		for (int j=0;j<MAX;j++)
		{
			cout <<setw(15)<<output[i][j];
		}
		cout <<endl;
	}
}

void printTensor(Real ***output, int MAX)
{
	for (int i=0;i<MAX;i++)
	{
		printArray(output[i], MAX);
		cout <<endl;
	}
}

void FFT2D(Real **InputReal, Real **InputImag, Real **OutputReal, Real **OutputImag, int max)
{
	//Only works for square matrix.
	Real **OutputRealTemp, **OutputImagTemp;

	OutputRealTemp=new Real *[max];
	OutputImagTemp=new Real *[max];
	for (int i=0;i<max;i++)
	{
		*(OutputRealTemp+i)=new Real[max];
		*(OutputImagTemp+i)=new Real[max];
	}

	for (int i=0;i<max;i++)
	{
		FFT(InputReal[i], InputImag[i], OutputRealTemp[i], OutputImagTemp[i], max);
	}
	transpose(OutputRealTemp, max, max);
	transpose(OutputImagTemp, max, max);	
	for (int i=0;i<max;i++)
	{
		FFT(OutputRealTemp[i], OutputImagTemp[i], OutputReal[i], OutputImag[i], max);
	}
	transpose(OutputReal, max, max);
	transpose(OutputImag, max, max);
	for (int i=0;i<max;i++)
	{
		delete [] OutputRealTemp[i];
		delete [] OutputImagTemp[i];
	}
	delete [] OutputRealTemp;
	delete [] OutputImagTemp;
}

void FFT3D(Real ***InputReal, Real ***InputImag, Real ***OutputReal, Real ***OutputImag, int max)
{
	//Only works for cubic input.
	Real ***OutputRealTemp, ***OutputImagTemp;

	OutputRealTemp=new Real **[max];
	OutputImagTemp=new Real **[max];
	for (int i=0;i<max;i++)
	{
		*(OutputRealTemp+i)=new Real *[max];
		*(OutputImagTemp+i)=new Real *[max];
		for (int j=0;j<max;j++)
		{
			*(*(OutputRealTemp+i)+j)=new Real[max];
			*(*(OutputImagTemp+i)+j)=new Real[max];
		}
	}

	for (int i=0;i<max;i++)
	{
		FFT2D(InputReal[i], InputImag[i], OutputRealTemp[i], OutputImagTemp[i], max);
	}
	//PrintTensor(OutputRealTemp, max);
	transpose(OutputRealTemp, max, max);
	//PrintTensor(OutputRealTemp, max);
	transpose(OutputImagTemp, max, max);	
	for (int i=0;i<max;i++)
	{
		transpose(OutputRealTemp[i], max, max);
		transpose(OutputImagTemp[i], max, max);
	}
	for (int i=0;i<max;i++)
	{
		for (int j=0;j<max;j++)
		{
			FFT(OutputRealTemp[i][j], OutputImagTemp[i][j], OutputReal[i][j], OutputImag[i][j], max);
		}
	}

	for (int i=0;i<max;i++)
	{
		transpose(OutputRealTemp[i], max, max);
		transpose(OutputImagTemp[i], max, max);
	}
	transpose(OutputReal, max, max);
	transpose(OutputImag, max, max);
	transpose2(OutputReal, max, max);
	transpose2(OutputImag, max, max);
	//transpose(OutputReal, max, max);
	//transpose(OutputImag, max, max);
	for (int i=0;i<max;i++)
	{
		for (int j=0;j<max;j++)
		{
			delete [] OutputRealTemp[i][j];
			delete [] OutputImagTemp[i][j];
		}
		delete [] OutputRealTemp[i];
		delete [] OutputImagTemp[i];
	}
	delete [] OutputRealTemp;
	delete [] OutputImagTemp;
}

void calcFTElement(Real ***InputReal, Real ***InputImag, Real &OutputReal, Real &OutputImag, int i, int j, int k, int max)
{
	Real theta;
	for (int a=0;a<max;a++)
	{
		for (int b=0;b<max;b++)
		{
			for (int c=0;c<max;c++)
			{
				theta=2.0*pi*Real(i*a+j*b+k*c)/Real(max);
				OutputReal+=InputReal[a][b][c]*cos(theta)-InputImag[a][b][c]*sin(theta);
				OutputImag+=InputImag[a][b][c]*cos(theta)+InputReal[a][b][c]*sin(theta);
			}
		}
	}
}

void FT3D(Real ***InputReal, Real ***InputImag, Real ***OutputReal, Real ***OutputImag, int max)
{
	for (int i=0;i<max;i++)
	{
		for (int j=0;j<max;j++)
		{
			for (int k=0;k<max;k++)
			{
				OutputReal[i][j][k]=0;
				OutputImag[i][j][k]=0;
				calcFTElement(InputReal, InputImag, OutputReal[i][j][k], OutputImag[i][j][k], i, j, k, max);
			}
		}
	}
}

void fitLine(vector<Real> &x, vector<Real> &y, Real &a, Real &b)
{
        //There may be something wrong with function.
        int Size;
        vector<Real> variable;
        vector< vector<Real> > coefficient;

        SafeAlloc(variable, 2, "variable in FitLine");
        Safe2DAlloc(coefficient, 3, 2, "coefficient in FitLine");
        Size=x.size();
        coefficient[0][0]=Real(Size);
        for (int i=0;i<Size;i++)
        {
                coefficient[1][0]+=x[i];
                coefficient[2][0]+=y[i];
                coefficient[1][1]+=x[i]*x[i];
                coefficient[2][1]+=x[i]*y[i];
        }
        coefficient[0][1]=coefficient[1][0];
        equation(coefficient, variable);

        a=variable[0];
        b=variable[1];
}

//template <typename T>
void multiVariableRegression(Vector &z, Matrix &data, Vector &variable)
{
        int nvariable=variable.size();
        int npoint=z.size();
        Matrix coefficient;

        Safe2DAlloc(coefficient, nvariable+1, nvariable, "in multiVariableRegression");
        //Print2DVectorSize(coefficient, "coefficeint");
        //Print2DVectorSize(data, "data");
        for (int i=0;i<nvariable-1;i++)
        {
                for (int j=0;j<nvariable-1;j++)
                {
                        for (int k=0;k<npoint;k++)
                        {
                                coefficient[i][j]+=data[i][k]*data[j][k];
                        }
                }
        }
	//cout <<"coefficient:"<<endl;
        //printMatrix(coefficient);
        //cout <<endl;
        for (int i=0;i<nvariable-1;i++)
        {
                for (int k=0;k<npoint;k++)
                {
                        coefficient[i][nvariable-1]+=data[i][k];
                }
        }
        //printMatrix(coefficient);
        //cout <<endl;
        for (int i=0;i<nvariable;i++)
        {
                for (int j=0;j<nvariable;j++)
                {
                        coefficient[j][i]=coefficient[i][j];
                }
        }
        coefficient[nvariable-1][nvariable-1]=Real(npoint);
        //printMatrix(coefficient);
        //cout <<endl;
        
        for (int i=0;i<nvariable-1;i++)
        {
                for (int j=0;j<npoint;j++)
                {
                        coefficient[nvariable][i]+=z[j]*data[i][j];
                }
        }
        for (int j=0;j<npoint;j++)
        {
                coefficient[nvariable][nvariable-1]+=z[j];
        }
        //printMatrix(coefficient);
        //cout <<endl;
        equation(coefficient, variable);
}

Real calcSxy(vector<Real> &x, vector<Real> &y, Real a, Real b)
{
        int Size;
        Real diff, sum, sxy;

        Size=x.size();

        sum=0;

        for (int i=0;i<Size;i++)
        {
                diff=(y[i]-a-b*x[i]);
                sum+=diff*diff;
        }
        sxy=sum/Real(Size-1);
        return sxy;
}

Real calcSy(vector<Real> &x, vector<Real> &y)
{
        int Size;
        Real AverageY, sum, diff, sy;

        Size=x.size();
        AverageY=calcAverage(y);

        sum=0;
        for (int i=0;i<Size;i++)
        {
                diff=y[i]-AverageY;
                sum+=diff*diff;
        }

        sy=sum/Real(Size-1);
        return sy;
}
/*
Real correlation(vector<Real> &x, vector<Real> &y)
{
        Real a, b;
        Real sxy, sy, r2;

        a=0;
        b=0;

        //FitLine(x, y, a, b);
        linearRegression(x, y, a, b);
        sxy=calcSxy(x, y, a, b);
        sy=calcSy(x, y);
        r2=1.0-sxy/sy;
	cout <<"a= "<<a<<" b= "<<b<<" sxy= "<<sxy<<" sy= "<<sy<<endl;
        if (b<0) r2=-r2;       
	cout <<"r2= "<<r2<<endl; 
        return r2;
}
*/

Real correlation(vector<Real> &x, vector<Real> &y)
{
	int Size=x.size();
	Real cov, xStdDev, yStdDev;

	cov=calcCovariance(x, y);
	xStdDev=calcStandardDeviation(x);
	yStdDev=calcStandardDeviation(y);

	return cov/(xStdDev*yStdDev*Real(Size));
}

Real correlation(vector<Real> &x, vector<Real> &y, Real a, Real b)
{
        Real sxy, sy, r2;

        sxy=calcSxy(x, y, a, b);
        sy=calcSy(x, y);
        r2=1.0-sxy/sy;
        if (b<0) r2=-r2;        
        return r2;
}

//template <typename T>
Real calcSxy(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable)
{
        int Size;
        int nvariable=variable.size();
        Real diff, sum, sxy;

        Size=z.size();

        sum=0;

        for (int i=0;i<Size;i++)
        {
                diff=z[i]-variable[nvariable-1];
                for (int j=0;j<nvariable-1;j++)
                {
                        diff-=variable[j]*data[j][i];
                }
                sum+=diff*diff;
        }
        sxy=sum/Real(Size-1);
        return sxy;
}

//template <typename T>
Real calcSy(vector<Real> &y)
{
        int Size;
        Real AverageY, sum, diff, sy;

        Size=y.size();
        AverageY=calcAverage(y);

        sum=0;
        for (int i=0;i<Size;i++)
        {
                diff=y[i]-AverageY;
                sum+=diff*diff;
        }

        sy=sum/Real(Size-1);
        return sy;
}

Real calcCorrelationFromFit(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable)
{
        Real sxy, sy, r2;
        sxy=calcSxy(z, data, variable);
        sy=calcSy(z);
        r2=1.0-sxy/sy;

        return r2;
}

Real linearRegression(vector<Real> &x, vector<Real> &y, Real &slope, Real &yintercept, Real &slopeError, Real &yError)
{
        Real sx, sxx, sy, syy, se2, sa2, sb2;
        Real Size=Real(x.size());
        linearRegression(x, y, slope, yintercept);
        sxx=calcSumSqr(x);
        sx=calcSum(x);
        syy=calcSumSqr(y);
        sy=calcSum(y);
        se2=(Size*syy-sy*sy-slope*slope*(Size*sxx-sx*sx))/(Size*(Size-2));
        sb2=Size*se2/(Size*sxx-sx*sx);
        sa2=sb2*sxx/Size;
        cout <<"sa2= "<<sa2<<endl;
        cout <<"sb2= "<<sb2<<endl;
        yError=sqrt(sa2);
        slopeError=sqrt(sb2);
        return correlation(x, y);
}

//template <typename T>
Real calcCorrelation(vector<Real> &z, vector< vector<Real> > &data, vector<Real> &variable)
{
        Real r2;

        multiVariableRegression(z, data, variable);
        r2=calcCorrelationFromFit(z, data, variable);

        return r2;
}

Real calcPearson(Vector &x, Vector &y)
{
	int xSize=x.size();
	int ySize=y.size();
	Real cov, xAve, yAve;
	Real xSum=0, ySum=0;

	if (xSize!=ySize || xSize==0)
	{
		string errorStr="xSize= "+toStr(xSize)+" ySize= "+toStr(ySize);
		error(errorStr, __LINE__, __FILE__);
	}
	cov=calcCovariance(x, y);

	xAve=calcAverage(x);
	yAve=calcAverage(y);
	for (int i=0;i<xSize;i++)
	{
		xSum+=(x[i]-xAve)*(x[i]-xAve);
		ySum+=(y[i]-yAve)*(y[i]-yAve);
	}
	return cov/(sqrt(xSum)*sqrt(ySum));
}

Real roundReal(Real x, Real nearest)
{
	if (x<0) return -nearest*Real(int(abs(x)/nearest+0.5));
        return nearest*Real(int(x/nearest+0.5));
}

bool isLastPermutation(vector<int> &permutation)
{
	bool first=false, second=false;
	int Size=permutation.size();

	if (permutation[Size-1]==0) return false;
	for (int i=Size-1;i>=0;i--)
	{
		if (permutation[i]==1) first=true;
		if (permutation[i]==0 && first) second=true;
		if (permutation[i]==1 && second) return false;
	}
	return true;
}

int findLast(vector<int> &permutation)
{
	int Size=permutation.size();

	for (int i=Size-1;i>=0;i--)
	{
		if (permutation[i]==1)
		{
			return i;
		}
	}
	return UNK_INT;
}

int findSecondToLast(vector<int> &permutation)
{
	bool firstZero=false;
	int Size=permutation.size();
	for (int i=Size-1;i>=0;i--)
	{
		if (permutation[i]==0)
		{
			firstZero=true;
		}
		if (permutation[i]==1 && firstZero)
		{
			return i;
		}
	}
	return UNK_INT;
}

int countNumAfter(vector<int> &permutation, int last)
{
	//I don't think this is right
	int count=0;
	int Size=permutation.size();

	for (int i=last+1;i<Size;i++)
	{
		if (permutation[i]==1) count++;
	}
	return count;
}

void getNextPermutation(vector<int> &permutation)
{
	int Size=permutation.size();
	int last, count;

	last=findLast(permutation);

	if (last<Size-1)
	{
		permutation[last]=0;
		permutation[last+1]=1;
	}
	else
	{
		last=findSecondToLast(permutation);
		count=countNumAfter(permutation, last);
		permutation[last]=0;
		for (int i=last+1;i<=last+1+count;i++)
		{
			permutation[i]=1;
		}
		for (int i=last+2+count;i<Size;i++)
		{
			permutation[i]=0;
		}
	}
}

Real nChooseK(int n, int k)
{
	Real sum=0;
	for (int i=n;i<n-k;i++) sum+=log(Real(i));
	for (int i=1;i<=k;i++) sum-=log(Real(i));
	return(exp(sum));
}

Real updatePrincipleComponent(Matrix &data, Vector &r)
{
	int Size1, Size2;
	Real dp, change;	
	Vector s, scaled;

	Get2DVectorSize(data, Size1, Size2, "data");
	SafeAlloc(s, Size2, "s");

	for (int i=0;i<Size1;i++)
	{
		dp=calcDotProduct(data[i], r);
		//cout <<"r:"<<endl;
		//printVector(r);
		//cout <<"data:"<<endl;
		//printVector(data[i]);
		scaled=multiply(dp, data[i]);		
		s=addVectors(s, scaled);
		//cout <<"dp= "<<dp<<endl;
		//cout <<"scaled:"<<endl;
		//printVector(scaled);
		//cout <<"s:"<<endl;
		//printVector(s);
		//cout <<endl;
		//endProgram(__LINE__, __FILE__);
	}
	setLength(s, 1.0);
	change=calcSqrDiff(s, r);
	r=s;
	return change;
}

void calcPrincipleComponent(Matrix &data, Vector &r)
{
	int maxIter=100;
	int Size1, Size2;
	Real change;
	Real convergenceCriterion=1.0e-9;

	Get2DVectorSize(data, Size1, Size2, "data");
	createRandomVector(r, Size2);
	for (int i=0;i<maxIter;i++)
	{
		change=updatePrincipleComponent(data, r);
		if (change<convergenceCriterion) break;
	}	
}

void removeAverage(Vector &v)
{
	int Size=v.size();
	Real ave;

	ave=calcAverage(v);
	for (int i=0;i<Size;i++) v[i]-=ave;
}

void removeAverage(Matrix &data)
{
	int Size1, Size2;
	Real ave;

	Get2DVectorSize(data, Size1, Size2, "data");

	for (int i=0;i<Size1;i++)
	{
		ave=calcAverage(data[i]);
		for (int j=0;j<Size2;j++) data[i][j]-=ave;
	}
}

Real calcPrincipleComponentWeight(Vector &data, Vector &r)
{
	return calcScaleForSqrDiff(r, data);
}

void subtractPrincipleComponent(Vector &data, Vector &r)
{
	int Size=data.size();
	Real scale;

	scale=calcPrincipleComponentWeight(data, r);

	for (int i=0;i<Size;i++)
	{
		data[i]-=scale*r[i];
	}
}

void subtractPrincipleComponent(Matrix &data, Vector &r)
{
	int Size=data.size();

	for (int i=0;i<Size;i++)
	{
		subtractPrincipleComponent(data[i], r);
	}
}

void checkOrthonormality(Matrix &pca)
{
	int Size=pca.size();
	Matrix dp;

	Safe2DAlloc(dp, Size, Size, "dp");
	for (int i=0;i<Size;i++)
	{
		for (int j=0;j<Size;j++)
		{
			dp[i][j]=calcDotProduct(pca[i], pca[j]);
		}
	}
	cout <<"dp: "<<endl;
	printMatrix(dp);
}

void calcPca(Matrix data, Matrix &pca)
{
	//Algorithm from https://en.wikipedia.org/wiki/Principal_component_analysis
	int Size1, Size2;
	Vector r;

	Get2DVectorSize(data, Size1, Size2, "data");
	//removeAverage(data);

	for (int i=0;i<Size2;i++)
	{
		calcPrincipleComponent(data, r);
		subtractPrincipleComponent(data, r);
		SafePushBack(pca, r, "pca");
	}
	//checkOrthonormality(pca);
}
