/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
//This is not as good as the original fortram TM-score program by Yang Zhang.  This has an R squared of 0.9997 to the older program.

# include <numeric>

# include "AtomUtils.h"
# include "Constants.h"
# include "Structures.h"
# include "TypeDef.h"
# include "minimize.h"
# include "ReadPdb.h"
# include "PrintPdb.h"
# include "rmsd.h"

struct TMScoreArgs
{
	string TMScoreType;
	vector<AtomStruct> Atoms1, Atoms2;
};

Real TMScoreNoAlign(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
        Real dist, TMScore=0, NumAve=0, d0, d02, inv;
        int natom, l;
        natom=Atoms1.size();
        //if (natom!=Atoms2.size()) 
        //{
        //	cout <<"Error the two molecules do not have the same number of atoms.";
        //	exit(0);
        //}
	l=getNumResidues(Atoms1);
	d0=1.24*power(Real(abs(l-15)), 1.0/3.0)-1.8;
	d02=d0*d0;
	inv=1.0/d02;
	//cout <<"d02= "<<d02<<endl;
        for (int i=0;i<natom;i++)
        {
                if (Atoms1[i].AtomName=="CA" || Atoms1[i].AtomName=="O1P")
                {
                        dist=atomSquareDistance(Atoms1[i], Atoms2[i]);
			TMScore+=1.0/(1.0+dist*inv);
                        NumAve+=1.0;
			//cout <<"TMScore= "<<TMScore<<" dist= "<<dist<<" inv= "<<inv<<" NumAve= "<<NumAve<<endl;
			//TMScore+=Atoms2[i].Occupancy/(1.0+dist*inv);
                        //NumAve+=Atoms2[i].Occupancy;
                }
        }
	//cout <<"NumAve= "<<NumAve<<endl;
        TMScore/=NumAve;
        return TMScore;
}

Real TMScoreNoAlignTaylor(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
        Real dist, TMScore=0, NumAve=0, d0, d02, inv;
        int natom, l;
        natom=Atoms1.size();
        //if (natom!=Atoms2.size()) 
        //{
        //	cout <<"Error the two molecules do not have the same number of atoms.";
        //	exit(0);
        //}
	l=getNumResidues(Atoms1);
	d0=1.24*power(Real(abs(l-15)), 1.0/3.0)-1.8;
	d02=d0*d0;
	inv=1.0/d02;
	//cout <<"d02= "<<d02<<endl;
        for (int i=0;i<natom;i++)
        {
                if (Atoms1[i].AtomName=="CA" || Atoms1[i].AtomName=="O1P")
                {
                        dist=atomSquareDistance(Atoms1[i], Atoms2[i]);
			if (dist*inv<0.01) TMScore+=(1-dist*inv);
			else TMScore+=1.0/(1.0+dist*inv);
                        NumAve+=1.0;
                }
        }
        TMScore/=NumAve;
        return TMScore;
}

Real TMScoreNoAlignAll(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
        Real dist, TMScore=0, NumAve=0, d0, d02, inv;
        int natom, l;
        natom=Atoms1.size();
        //if (natom!=Atoms2.size()) 
        //{
        //	cout <<"Error the two molecules do not have the same number of atoms.";
        //	exit(0);
        //}
	l=getNumResidues(Atoms1);
	d0=1.24*power(Real(l), 1.0/3.0)-1.8;
	d02=d0*d0;
	inv=1.0/d02;
        for (int i=0;i<natom;i++)
        {
		dist=atomSquareDistance(Atoms1[i], Atoms2[i]);
		TMScore+=1.0/(1.0+dist*inv);
		NumAve+=1.0;
        }
        TMScore/=NumAve;
        return TMScore;
}

Real TMScoreNoAlignAverage(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
        int natom, l, Size=5;
        Real dist, TMScore=0, NumAve=0, d0, d02;
	Vector invs;
        natom=Atoms1.size();
        //if (natom!=Atoms2.size()) 
        //{
        //	cout <<"Error the two molecules do not have the same number of atoms.";
        //	exit(0);
        //}
	l=getNumResidues(Atoms1);
	d0=1.24*power(Real(l), 1.0/3.0)-1.8;
	d02=d0*d0;
	for (int i=0;i<Size;i++)
	{
		SafePushBack(invs, Real(i+1)/(d02*Real(Size)), "invs");
	}
        for (int i=0;i<natom;i++)
        {
		dist=atomSquareDistance(Atoms1[i], Atoms2[i]);
		for (int j=0;j<Size;j++) TMScore+=1.0/(1.0+dist*invs[j]);
		NumAve+=1.0;
        }
        TMScore/=(NumAve*Real(Size));
        return TMScore;

}

void placeAtoms(vector<AtomStruct> &Atoms, Vector &coefficient)
{
	//RotateAtoms(Atoms, coefficient[X], coefficient[Y], coefficient[Z]);
	rotateAtomsAroundVector(Atoms, coefficient[0], coefficient[1], coefficient[2]);
	moveAtoms(Atoms, coefficient[3+X], coefficient[3+Y], coefficient[3+Z]);
}

Real evaluateAlignment(vector<AtomStruct> &Atoms1, vector<AtomStruct> Atoms2, Vector &coefficient, string TMScoreType)
{
	placeAtoms(Atoms2, coefficient);
        if (TMScoreType=="Normal") 
	{
		return -TMScoreNoAlign(Atoms1, Atoms2);
	}
        else if (TMScoreType=="Average") 
	{
		return -TMScoreNoAlignAverage(Atoms1, Atoms2);
	}
	else
	{
		string errorStr=TMScoreType+" is not a valid TMScoreType.  The ";
		errorStr+=" valid TMScoreTypes are Normal and Average";
		error(errorStr, __LINE__, __FILE__);
	}
	return 0;
}

Real evaluateAlignment(Vector coefficients, TMScoreArgs args)
{
	return evaluateAlignment(args.Atoms1, args.Atoms2, coefficients, args.TMScoreType);
}

Real GridSearch(vector<AtomStruct> Atoms1, vector<AtomStruct> Atoms2, Real &theta, Real &phi, Real &psi, string TMScoreType)
{
        Real BestTheta, BestPhi, BestPsi;
        Real BestRmsd, rmsd;
        Real ThetaInc, PhiInc, PsiInc;
	Vector coefficients;

	theta=phi=psi=0;
	SafeAlloc(coefficients, 6, "coefficients");
        ThetaInc=2.0*pi/10.0;
        PhiInc=2.0*pi/10.0;
        PsiInc=2.0*pi/10.0;
        BestTheta=theta; BestPhi=phi; BestPsi=psi;
        BestRmsd=evaluateAlignment(Atoms1, Atoms2, coefficients, TMScoreType);
        for (theta=0;theta<2.0*pi;theta+=ThetaInc)
        {
                for (phi=0;phi<2.0*pi;phi+=PhiInc)
                {
                        for (psi=0;psi<2.0*pi;psi+=PsiInc)
                        {
				coefficients[X]=theta;
				coefficients[Y]=phi;
				coefficients[Z]=psi;
                                rmsd=evaluateAlignment(Atoms1, Atoms2, coefficients, TMScoreType);
                                //cout <<"rmsd= "<<rmsd<<" psi= "<<psi<<endl;
                                if (rmsd<BestRmsd)
                                {
                                        BestRmsd=rmsd; BestTheta=theta; BestPhi=phi; BestPsi=psi;
                                        //cout <<"rmsd= "<<rmsd<<" theta= "<<theta<<" phi= "<<phi<<" psi= "<<psi<<endl;
                                }
                        }
                }
        }
        theta=BestTheta; phi=BestPhi, psi=BestPsi;
        return BestRmsd;
}

Real calcTMScore(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, string TMScoreType, Real &theta, Real &phi, Real &psi, Real &dx, Real &dy, Real &dz)
{
	int natom=Atoms1.size();
        Real dScoreCriteria=10e-12;
        Real TMScore;
        Real XOrigin1, YOrigin1, ZOrigin1;
        Real XOrigin2, YOrigin2, ZOrigin2;
	Real maxHours=0.012;
	//Real maxHours=0.012;
        Vector coefficients;
	vector<bool> use;
        vector<AtomStruct> CopyAtoms1, CopyAtoms2;
	TMScoreArgs args;
	Real (*func_to_minimize)(Vector, TMScoreArgs)=evaluateAlignment;

	
	SafeAlloc(use, true, natom, "use");
	SafeAlloc(coefficients, 6, "coefficients");
        CopyVector(Atoms1, CopyAtoms1);
        CopyVector(Atoms2, CopyAtoms2);
        removeWaters(CopyAtoms1);
        removeWaters(CopyAtoms2);
        theta=phi=psi=0;
        calcCenter(CopyAtoms1, XOrigin1, YOrigin1, ZOrigin1);
        calcCenter(CopyAtoms2, XOrigin2, YOrigin2, ZOrigin2);
	dx=XOrigin1-XOrigin2;
	dy=YOrigin1-YOrigin2;
	dz=ZOrigin1-ZOrigin2;
        center(CopyAtoms1);
        center(CopyAtoms2);
	args.Atoms1=CopyAtoms1;
	args.Atoms2=CopyAtoms2;
	args.TMScoreType=TMScoreType;
        GridSearch(CopyAtoms1, CopyAtoms2, theta, phi, psi, TMScoreType);
	cout <<"theta= "<<theta<<endl;
	cout <<"phi= "<<phi<<endl;
	cout <<"psi= "<<psi<<endl;
	//RMSD(CopyAtoms1, CopyAtoms2, theta, phi, psi, use);
	coefficients[X]=theta;
	coefficients[Y]=phi;
	coefficients[Z]=psi;
	//placeAtoms(CopyAtoms2, coefficients);
	//TMScore_old=TMScoreNoAlign(CopyAtoms1, CopyAtoms2);
        //TMScore=TMScore_old;
	TMScore=optimizeCoefficients(coefficients, args, func_to_minimize, dScoreCriteria, maxHours);	
	theta=coefficients[X];
	phi=coefficients[Y];
	psi=coefficients[Z];
	dx+=coefficients[X+3];
	dy+=coefficients[Y+3];
	dz+=coefficients[Z+3];
	//TMScore=conjugateGradient(coefficients, args, func_to_minimize, dScoreCriteria, maxHours);	
	//Atoms2=CopyAtoms2;
        //center(Atoms2);
	printVector(coefficients, "coefficients");
	placeAtoms(CopyAtoms2, coefficients);
	//moveAtoms(CopyAtoms2, XOrigin1, YOrigin1, ZOrigin1);
	TMScore=TMScoreNoAlign(CopyAtoms1, CopyAtoms2);
	//cout <<"TMScore= "<<TMScore_old<<endl;
        cout <<"TMScore= "<<TMScore<<endl;
        return TMScore;

}

Real calcTMScore(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2, string TMScoreType)
{

        Real theta=0, phi=0, psi=0;
	Real dx, dy, dz;
	Real TMScore;

	TMScore=calcTMScore(Atoms1, Atoms2, TMScoreType, theta, phi, psi, dx, dy, dz);
	rotateAtomsAroundVector(Atoms2, theta, phi, psi);
	moveAtoms(Atoms2, dx, dy, dz);
	cout <<"TMScore= "<<TMScoreNoAlign(Atoms1, Atoms2)<<endl;
	return TMScore;
}

Real calcTMScore(vector<AtomStruct> &Atoms1, vector<AtomStruct> &Atoms2)
{
	return calcTMScore(Atoms1, Atoms2, "Normal");
}

Real calcTMScore_CA(vector<AtomStruct> Atoms1, vector<AtomStruct> Atoms2)
{
        removeNonCA(Atoms1);
        removeNonCA(Atoms2);
        cout <<"Atoms1.size()= "<<Atoms1.size()<<endl;
        cout <<"Atoms2.size()= "<<Atoms2.size()<<endl;
        return calcTMScore(Atoms1, Atoms2);
}

Real calcTMScore_CA(string pdbFile, vector<AtomStruct> Atoms2)
{
	vector<AtomStruct> Atoms1;

	readPdb(pdbFile, Atoms1);
	removeNonCA(Atoms1);
	removeNonCA(Atoms2);
	return calcTMScore(Atoms1, Atoms2);
}

