/*
    JV Library
    Copyright (C) 2014  Jouko Juhani Virtanen

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
#ifndef _minimize_included_
#define _minimize_included_

# include <sys/time.h>

# include "TypeDef.h"
# include "MathUtils.h"
# include "VectorManip.h"
# include "test_functions.h"
# include "time.h"
        
        template<class Args>
Real minimizeAlongEstimatedGrad(Vector &coefficient, Matrix &hessian, Vector &derivative, Args args, Real (*func_to_minimize)(Vector, Args));

	template <class Args>
bool AtMaxima(Vector coefficient, Real &TestStep, int index, Args args, Real (*func_to_minimize)(Vector, Args))
{
        Real score1, score2, score3;

        coefficient[index]-=TestStep;
        score1=(*func_to_minimize)(coefficient, args);
        coefficient[index]+=TestStep;
        score2=(*func_to_minimize)(coefficient, args);
        coefficient[index]+=TestStep;
        score3=(*func_to_minimize)(coefficient, args);
        coefficient[index]-=TestStep;

        if (score2>score1 && score2>score3) return true;
        return false;
}

        template <class Args>
bool AtMinima(Vector coefficient, Real &TestStep, int index, Args args, Real (*func_to_minimize)(Vector, Args))
{
        Real score1, score2, score3;

        coefficient[index]-=TestStep;
        score1=(*func_to_minimize)(coefficient, args);
        coefficient[index]+=TestStep;
        score2=(*func_to_minimize)(coefficient, args);
        coefficient[index]+=TestStep;
        score3=(*func_to_minimize)(coefficient, args);
        coefficient[index]-=TestStep;

        if (score2<score1 && score2<score3) return true;
        return false;
}

        template <class Args>
Real CalcDerivative(Vector coefficient, Real &TestStep, int index, Args &args, Real (*func_to_minimize)(Vector, Args))
{
        Real derivative, dScore;
        Real score, score_old;

        if (TestStep==0) TestStep=10e-6;
        score_old=(*func_to_minimize)(coefficient, args);
        coefficient[index]+=TestStep;
        score=(*func_to_minimize)(coefficient, args);
        dScore=score-score_old;
        derivative=dScore/TestStep;
        //cout <<"derivative= "<<derivative<<endl;
        return derivative;
}

        template <class Args>
Real CalcDerivative(Vector coefficient, Real &TestStep, Real score_old, int index, Args &args, Real (*func_to_minimize)(Vector, Args))
{
        Real derivative, dScore;
        Real score;

        if (TestStep==0) TestStep=10e-6;
        coefficient[index]+=TestStep;
        score=(*func_to_minimize)(coefficient, args);
        dScore=score-score_old;
        derivative=dScore/TestStep;
        //cout <<"derivative= "<<derivative<<endl;
        return derivative;
}
	
	template <class Args>
Real CalcDerivative(Vector &coefficient, Real &TestStep1, Real &TestStep2, Real score_old, int index, Args &args, Real (*func_to_minimize)(Vector, Args), int numTry)
{
	int maxTry=10; //If derivative calculation takes maxTry tries, give up.
        Real derivative=0, derivative1, derivative2;
	//Check derivative in two diretions to ensure numerical stability.		
	derivative1=CalcDerivative(coefficient, TestStep1, score_old, index, args, func_to_minimize);
	derivative2=CalcDerivative(coefficient, TestStep2, score_old, index, args, func_to_minimize);
	//Good enough if signs agree.
	if ((derivative1>0 && derivative2>0) || (derivative1<0 && derivative2<0))
	{
		derivative=(derivative1+derivative2)*0.5;
		return derivative;
	}
	numTry++;
	if (numTry>=maxTry)
	{
		return 0;
	}
	//If signs are different reduce step sizes and recalculate.
	if ((derivative1>0 && derivative2<0) || (derivative1<0 && derivative2>0))
	{
		TestStep1*=0.05;
		TestStep2*=0.05;
		derivative=CalcDerivative(coefficient, TestStep1, TestStep2, score_old, index, args, func_to_minimize, numTry);
		return derivative;
	}
	//If one derivative is 0 increase step size.
	if (derivative1==0 || derivative2==0)
	{
		numTry++;
		if (derivative1==0) 
		{
			TestStep1*=1.1;
			derivative1=CalcDerivative(coefficient, TestStep1, score_old, index, args, func_to_minimize);
		}
		if (derivative2==0) 
		{
			TestStep2*=1.1;
			derivative2=CalcDerivative(coefficient, TestStep2, score_old, index, args, func_to_minimize);
		}
		if ((derivative1>0 && derivative2>0) || (derivative1<0 && derivative2<0)) 
		{
			return (derivative1+derivative2)*0.5;
		}
		else if (derivative1>0 && derivative2<0) return (derivative1+derivative2)*0.5;  //If second derivative is negative return average
		else if (derivative1<0 && derivative2>0) return 0;				//If second derivative is positive return 0
		else if (derivative1==0 || derivative2==0) 					//If one derivative is 0 repeat
		{
			derivative=CalcDerivative(coefficient, TestStep1, TestStep2, score_old, index, args, func_to_minimize, numTry);
			return derivative;
		}
	}
        return derivative;
}

	template <class Args>
Real CalcDerivative(Vector &coefficient, int index, Real score_old, Args &args, Real (*func_to_minimize)(Vector, Args))
{
	int numTry=0;
	Real derivative;
	Real TestStep1=coefficient[index]*1e-6;
	Real TestStep2;

	if (TestStep1==0) TestStep1=1e-6;
	TestStep2=-TestStep1;
	derivative=CalcDerivative(coefficient, TestStep1, TestStep2, score_old, index, args, func_to_minimize, numTry);
	return derivative;
}
        
	template <class Args>
Real CalcGradient(Vector &coefficient, Args &args, Real (*func_to_minimize)(vector<Real>, Args), Vector &derivative)
{
        int Size;
        Real score_old;
	timeval start, end;
        Size=coefficient.size();
        //for (n=0;n<Size;n++) cout <<"coefficient["<<n<<"]= "<<coefficient[n]<<endl;
	gettimeofday(&start, NULL);
        score_old=(*func_to_minimize)(coefficient, args);
        for (int n=0;n<Size;n++)
        {
                //cout <<"n= "<<n<<endl;
                derivative[n]=CalcDerivative(coefficient, n, score_old, args, func_to_minimize);
                //cout <<"derivative["<<n<<"]= "<<derivative[n]<<endl;
		if (isnan(derivative[n]))
		{
			cout <<"WARNING: derivative["<<n<<"]= nan"<<endl;
			derivative[n]=0;
		}
        }
	gettimeofday(&end, NULL);
	//cout <<"CalcGradient took "<<calcTimeDiff(start, end)<<endl;
        return 0.1;
}

        template <class Args>
Real calcHessianElement(Vector coefficient, Args args, Real (*func_to_minimize)(Vector, Args), int index1, int index2)
{
        Real derivative1, derivative2;
        Real TestStep1, TestStep2;
	Real score_old;
        TestStep1=coefficient[index1]*10e-6;
        TestStep2=TestStep1*0.1;
	if (TestStep2==0) TestStep2=0.00001;
        score_old=(*func_to_minimize)(coefficient, args);
        derivative1=CalcDerivative(coefficient, index1, score_old, args, func_to_minimize);
        coefficient[index2]+=TestStep2;
        derivative2=CalcDerivative(coefficient, index1, score_old, args, func_to_minimize);
        return (derivative2-derivative1)/TestStep2;
}

        template <class Args>
Real calcHessianElement(Vector coefficient, Args &args, Real (*func_to_minimize)(Vector, Args), Real derivative1, Real score_old, int index1, int index2)
{
        Real derivative2;
        Real testStep;
        testStep=coefficient[index1]*10e-7;
	if (testStep==0) testStep=0.00001;
        coefficient[index2]+=testStep;
        derivative2=CalcDerivative(coefficient, index1, score_old, args, func_to_minimize);
        return (derivative2-derivative1)/testStep;
}

        template <class Args>
void calcHessian(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), Matrix &hessian)
{
        int Size=coefficient.size();

        for (int i=0;i<Size;i++)
        {
                for (int j=i;j<Size;j++)
                {
                        hessian[i][j]=calcHessianElement(coefficient, args, func_to_minimize, i, j);
                        hessian[j][i]=hessian[i][j];
                }
        }
}

        template <class Args>
void calcHessianAndGrad(vector<Real> coefficient, Args &args, Real (*func_to_minimize)(vector<Real>, Args), Matrix &hessian, Vector &derivative)
{
        int Size=coefficient.size();
	Real score_old;

        score_old=(*func_to_minimize)(coefficient, args);
	CalcGradient(coefficient, args, func_to_minimize, derivative);
        for (int i=0;i<Size;i++)
        {
                for (int j=i;j<Size;j++)
                {
                        hessian[i][j]=calcHessianElement(coefficient, args, func_to_minimize, derivative[i], score_old, i, j);
                        hessian[j][i]=hessian[i][j];
                }
        }
}

        template <class Args>
void calcHessianDiagonal(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), vector<Real> &secondDerivative)
{
        int Size=coefficient.size();

        for (int i=0;i<Size;i++)
        {
        	secondDerivative[i]=calcHessianElement(coefficient, args, func_to_minimize, i, i);
        }
}

        template <class Args>
void calcStepForConjugateGradient(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), vector<Real> &step)
{
        int Size=step.size();
        vector<Real> derivative;
        Matrix hessian;
        SafeAlloc(derivative, Size, "derivative");
        Safe2DAlloc(hessian, Size, Size, "hessian");
        calcHessianAndGrad(coefficient, args, func_to_minimize, hessian, derivative);
        step=solveHessian(derivative, hessian);
	for (int i=0;i<Size;i++)
	{
		if (isnan(step[i]))
		{
			cout <<endl;
			printVector(derivative, "derivative");
			cout <<endl;
			cout <<"hessian"<<endl;
			PrintHessian(hessian);
			step=derivative;
			cout <<"WARNING: Step is nan"<<endl;
			//error("Step is nan", __LINE__, __FILE__);
		}
	}
	cout <<"Leaving calcStepForConjugateGradient"<<endl;
}

        template <class Args>
Real calcGradientMagnitude(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), vector<Real> &derivative)
{
        CalcGradient(coefficient, args, func_to_minimize, derivative);
        return calcVectorMagnitude(derivative);
}

        template <class Args>
void calcStepForSteepestDescent(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), vector<Real> &step)
{
	int Size=step.size();
        Real length;
        length=CalcGradient(coefficient, args, func_to_minimize, step);
        if (length!=0) scaleVector(step, length);
	for (int i=0;i<Size;i++)
	{
		if (isnan(step[i])) step[i]=0;
	}
}

        template<class Args>
int CountMaxima(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args))
{
        int NumMaxima, Size;
        Real TestStep;
        bool Maxima;
        //cout <<"In CountMaxima"<<endl;
        Size=coefficient.size();
        NumMaxima=0;
        for (int n=0;n<Size-1;n++)
        {
                //cout <<"n= "<<n<<endl;
                TestStep=coefficient[n]*0.001;
                Maxima=AtMaxima(coefficient, TestStep, n, args, func_to_minimize);
                if (Maxima) NumMaxima++;
        }
        return NumMaxima;
}

        template<class Args>
int CountMinima(vector<Real> coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args))
{
        int NumMinima, Size;
        Real TestStep;
        bool Minima;

        Size=coefficient.size();
        NumMinima=0;
        for (int n=0;n<Size;n++)
        {
                TestStep=coefficient[n]*1e-3;
                Minima=AtMinima(coefficient, TestStep, n, args, func_to_minimize);
                if (Minima) NumMinima++;
        }
        return NumMinima;
}

        template<class Args>
Real minimizeAlongVector(vector<Real> &coefficient, vector<Real> step, Args args, Real (*func_to_minimize)(vector<Real>, Args), int maxSteps)
{
        bool verbose=false;
        int Size1, Size2, numSteps=0, minSteps=10;
        int skip=1;
        Real dScoreCriteria;
        Real ConvergedScore;
        Real score, score_old, dScore;
        Real TotalStepSqr;
	Real fractionCriterion=1e-5;
        vector<Real> BestCoefficient;
        if (verbose) cout <<"In minimizeAlongVector"<<endl;
        dScoreCriteria=1e-15;
        Size1=coefficient.size();
	Size2=step.size();

	if (Size1!=Size2 || Size1==0)
	{
		string errorStr="Size1= "+toStr(Size1)+" Size2= "+toStr(Size2);
		error(errorStr, __LINE__, __FILE__);
	}

        CreateVector(BestCoefficient, Size1);
        BestCoefficient=coefficient;
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) 
	{
		cout <<"score_old= "<<score_old<<endl;
		for (int i=0;i<Size1;i++) cout <<"step["<<i<<"]= "<<step[i]<<endl;
	}
        ConvergedScore=score_old;	
        while (true)
        {
		for (int n=0;n<Size1;n++)
		{
			if (isnan(step[n]))
			{
				cout <<"WARNING: step["<<n<<"]= nan"<<endl;
				step[n]=0;
			}
		}
                for (int n=0;n<Size1;n++) coefficient[n]+=step[n];

                //for (int n=0;n<Size;n++) cout <<"step["<<n<<"]= "<<step[n]<<"\t"<<"coefficient= "<<"\t"<<coefficient[n]<<endl;
                //cout <<endl;
                //cout <<"Incremented coefficient"<<endl;
                score=(*func_to_minimize)(coefficient, args);
                dScore=score-score_old;
                //for (n=0;n<Size;n++) cout <<"score= "<<score<<"\t"<<"coefficient["<<n<<"]= "<<coefficient[n]<<"\t"<<" step["<<n<<"]= "<<step[n]<<endl;
                //cout <<"score= "<<score<<"\t"<<"coefficient[0]= "<<coefficient[0]<<"\t"<<" step[0]= "<<step[0]<<endl;
                //cout <<"score= "<<score<<endl;
                if (score>score_old)
                {
                        for (int n=0;n<Size1;n++)
                        {
                                coefficient[n]-=step[n];
                                //cout <<"step["<<n<<"]= "<<step[n]<<" about to decrease step"<<endl;
                                step[n]=-step[n]*0.5;
                                //cout <<"step["<<n<<"]= "<<step[n]<<" just decreased step"<<endl;
                                coefficient[n]=BestCoefficient[n];
                        }
                }
                else
                {
                        score_old=score;
                        for (int n=0;n<Size1;n++)
                        {
                                //cout <<"step["<<n<<"]= "<<step[n]<<" about to increase step"<<endl;
                                step[n]=step[n]*1.2;
                                //cout <<"step["<<n<<"]= "<<step[n]<<" just to increased step"<<endl;
                                BestCoefficient[n]=coefficient[n];
                        }
                }
                //cout <<"Updated step"<<endl;
                TotalStepSqr=0;
                for (int n=0;n<Size1;n++)
                {
                        TotalStepSqr+=step[n]*step[n];
                }
                //if (verbose) cout <<"TotalStepSqr= "<<TotalStepSqr<<"\t"<<"dScore2= "<<dScore*dScore<<"\t"<<"eval= "<<dScore*dScore/TotalStepSqr<<"\t"<<"crit= "<<dScoreCriteria<<endl;
                if (TotalStepSqr==0) break;
                //dScoreSqr=dScore*dScore;
                if (numSteps%skip==0 && verbose) cout <<"dScore= "<<dScore<<" dScore2/TotalStepSqr= "<<dScore*dScore/TotalStepSqr<<" dScoreCriteria= "<<dScoreCriteria<<" dConverged= "<<score-ConvergedScore<<" numSteps= "<<numSteps<<endl;
                //if (dScore*dScore/TotalStepSqr<dScoreCriteria && numSteps>minSteps && dScore!=0)
                if (dScore*dScore/TotalStepSqr<dScoreCriteria && numSteps>minSteps)
                {
                        //cout <<"About to enter CalcGradient"<<endl;
                        break;
                }
                if (numSteps>maxSteps) break;
                if (dScore==0) dScoreCriteria*=2.0;
		if (abs(dScore)/abs(score-ConvergedScore)<fractionCriterion) break;
                numSteps++;
        }
        coefficient=BestCoefficient;
        return score_old;
}

        template<class Args>
Real minimizeAlongVector(vector<Real> &coefficient, vector<Real> step, Args args, Real (*func_to_minimize)(vector<Real>, Args))
{
	int maxSteps=100000;
	return minimizeAlongVector(coefficient, step, args, func_to_minimize, maxSteps);
}

        template<class Args>
Real minimizeAlongEstimatedGrad(vector<Real> &coefficient, Matrix &hessian, Vector &derivative, Args args, Real (*func_to_minimize)(vector<Real>, Args))
{
	//Takes in hessian and gradient at one point, estimates gradient as it moves along, and minimizes the function.
        bool verbose=true;
        int Size1;
        Real convergedScore;
	Real length=0;
        Real score, score_old, dScore;
	Real fractionCriterion=1e-5;
        Vector bestCoefficient, step, start;
        if (verbose) cout <<"In minimizeAlongEstimatedGrad"<<endl;
        Size1=coefficient.size();
	
        start=bestCoefficient=coefficient;
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"score_old= "<<score_old<<endl;
	step=derivative;
	for (int i=0;i<Size1;i++) length+=abs(derivative[i]/hessian[i][i]);
	length=sqrt(length)*0.2;
	if (isnan(length)) length=0.0001;
	scaleVector(step, -length);
        convergedScore=score_old;	
        while (true)
        {
		for (int n=0;n<Size1;n++)
		{
			if (isnan(step[n]))
			{
				cout <<"WARNING: step["<<n<<"]= nan"<<endl;
				step[n]=0;
			}
		}
                for (int n=0;n<Size1;n++) coefficient[n]+=step[n];

                score=(*func_to_minimize)(coefficient, args);
                dScore=score-score_old;
                while (score>score_old)
                {
                        for (int n=0;n<Size1;n++) step[n]=-step[n]*0.5;
			length*=0.5;
                        coefficient=bestCoefficient;
			for (int n=0;n<Size1;n++) coefficient[n]+=step[n];
                	score=(*func_to_minimize)(coefficient, args);
                	dScore=score-score_old;
			if (abs(dScore)/abs(score-convergedScore)<fractionCriterion) break;
                }
                if (score<score_old)
                {
                        score_old=score;
			length*=1.2;
			bestCoefficient=coefficient;
                }
		if (score==convergedScore) break;
		if (abs(dScore)/abs(score-convergedScore)<fractionCriterion) break;
		estimateGradient(hessian, derivative, start, coefficient, step);
		scaleVector(step, -length);
        }
        coefficient=bestCoefficient;
	if (verbose) cout <<"Leaving minimizeAlongEstimatedGrad"<<endl;
        return score_old;
}

        template<class Args>
Real optimizeUsingEstimatedGrad(vector<Real> &coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), Real dScoreCriteria, Real maxHours)
{
        bool verbose=true;
        int  Size;
        Real score, score_old, dScore;
        Vector step, derivative;
	Matrix hessian;
	timeval start, end;

	if (verbose) cout <<"In optimizeUsingEstimatedGrad"<<endl;
	gettimeofday(&start, NULL);
        Size=coefficient.size();
        SafeAlloc(step, Size, "step");
        SafeAlloc(derivative, Size, "derivative");
	Safe2DAlloc(hessian, Size, Size, "hessian");
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"score_old= "<<score_old<<endl;
        calcHessianAndGrad(coefficient, args, func_to_minimize, hessian, derivative);
        if (verbose) cout <<"Done with CalcGradient"<<endl;
        while (true)
        {
                score=minimizeAlongEstimatedGrad(coefficient, hessian, derivative, args, func_to_minimize);
                if (verbose) cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                score_old=score;
		gettimeofday(&end, NULL);
		if (verbose) cout <<"time= "<<calcTimeDiff(start, end)/3600.0<<" maxHours= "<<maxHours<<endl;
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
		else if (maxHours<calcTimeDiff(start, end)/3600.0) break;
                else if (abs(dScore)<dScoreCriteria) break;
        	calcHessianAndGrad(coefficient, args, func_to_minimize, hessian, derivative);
        }

        for (int n=0;n<Size;n++)
        {
                if (verbose) cout <<"coefficient["<<n<<"]= "<<coefficient[n]<<endl;
        }
	if (verbose) cout <<"score_minimized= "<<score<<endl;
        return score;

}

        template<class Args>
void iterateOverCoefficients(vector<Real> &coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args))
{
        bool verbose=true;
        int Size=coefficient.size();
        vector<Real> step;
        if (verbose) cout <<"In IterateOverCoefficients"<<endl;
        CreateVector(step, Size);
        for (int i=0;i<Size;i++)
        {
                for (int j=0;j<Size;j++) step[j]=0;
                //step[i]=CalcDerivative(coefficient, i, args, func_to_minimize);
		step[i]=0.001;
                minimizeAlongVector(coefficient, step, args, func_to_minimize);
        }
}

        template<class Args>
void minimizeOneAtATime(vector<Real> &coefficient, Args args, Real (*func_to_minimize)(vector<Real>, Args), Real maxHours)
{
        bool verbose=false;
        int Size, NumMinima, n;
        Real score, score_old;
        Real dScoreCriterion;
        vector<Real> derivative;
	timeval start, end;

	gettimeofday(&start, NULL);
        n=0;
        dScoreCriterion=1e-5;
        Size=coefficient.size();
        CreateVector(derivative, Size);
        score=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"In MinimizeOneAtATime"<<endl;
        while (true)
        {
                score_old=score;
                iterateOverCoefficients(coefficient, args, func_to_minimize);
                //TotalGradient=calcGradientMagnitude(coefficient, args, func_to_minimize, derivative);
                score=(*func_to_minimize)(coefficient, args);
                NumMinima=CountMinima(coefficient, args, func_to_minimize);
                if (n%1000==0 && verbose)
                {
                        //cout <<"score_old= "<<score_old<<" score= "<<score<<" score_old-score= "<<score_old-score<<endl;
                        //cout <<"TotalGradient= "<<TotalGradient<<endl;
                        //cout <<"NumMinima= "<<NumMinima<<endl;
                }
		gettimeofday(&end, NULL);
		if (maxHours<calcTimeDiff(start, end)) break;
                if (score_old-score<dScoreCriterion) break;
                //if (TotalGradient<GradientCriterion) break;
                if (NumMinima==Size) break;
                n++;
        }
}

        template<class Args, class GradArgs>
Real nonlinearConjugateGradient(vector<Real> &coefficient, Args &args, GradArgs &gradArgs, Real (*func_to_minimize)(vector<Real>, Args), Real (*func_grad)(vector<Real>, GradArgs, vector<Real> &), Real dScoreCriteria, Real maxHours)
{
        //Algorithm from en.wikipedia.org/wiki/Energy_minimization
        int Size=coefficient.size();
        Real dScore, score, score_old, length=0.001;
        vector <Real> step, step_old, derivative, derivative_old;
	cout <<"Before score_old"<<endl;
        score_old=(*func_to_minimize)(coefficient, args);
	cout <<"score_old= "<<score_old<<endl;
	timeval start, end;

	gettimeofday(&start, NULL);
        SafeAlloc(step, Size, "step");
        SafeAlloc(derivative, Size, "derivative");
        SafeAlloc(step_old, Size, "step_old");
	cout <<"Before CalcGradient"<<endl;
	(*func_grad)(coefficient, gradArgs, derivative);
	cout <<"After CalcGradient"<<endl;
        step=derivative;
	scaleVector(step, length);
        while (true)
        {
                score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                //cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0)
                {
                        zeroVector(step_old);
			(*func_grad)(coefficient, gradArgs, derivative);
                        step=derivative;
                        score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                }
                dScore=score-score_old;
                score_old=score;
		gettimeofday(&end, NULL);
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
                else if (abs(dScore)<dScoreCriteria) break;
		cout <<"time= "<<calcTimeDiff(start, end)/3600.0<<endl;
		if (calcTimeDiff(start, end)/3600.0>maxHours) break;
                derivative_old=derivative;
		length=(*func_grad)(coefficient, gradArgs, derivative);
                updateStep(derivative, derivative_old, step, step_old);
                step_old=step;
                scaleVector(step, length);
        }
        return score;
}

        template<class Args>
Real nonlinearConjugateGradient(vector<Real> &coefficient, Args &args, Real (*func_to_minimize)(vector<Real>, Args), Real dScoreCriteria, Real maxHours)
{
        //Algorithm from en.wikipedia.org/wiki/Energy_minimization
        int Size=coefficient.size();
        Real dScore, score, score_old, length=0.1;
        vector <Real> step, step_old, derivative, derivative_old;
	cout <<"Before score_old"<<endl;
        score_old=(*func_to_minimize)(coefficient, args);
	cout <<"score_old= "<<score_old<<endl;
	timeval start, end;

	gettimeofday(&start, NULL);
        SafeAlloc(step, Size, "step");
        SafeAlloc(derivative, Size, "derivative");
        SafeAlloc(step_old, Size, "step_old");
	cout <<"Before CalcGradient"<<endl;
        CalcGradient(coefficient, args, func_to_minimize, derivative);
	cout <<"After CalcGradient"<<endl;
        step=derivative;
	scaleVector(step, length);
        while (true)
        {
                score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                //cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0)
                {
                        zeroVector(step_old);
                        CalcGradient(coefficient, args, func_to_minimize, derivative);
                        step=derivative;
                        score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                }
                dScore=score-score_old;
                score_old=score;
		gettimeofday(&end, NULL);
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
                else if (abs(dScore)<dScoreCriteria) break;
		cout <<"time= "<<calcTimeDiff(start, end)/3600.0<<endl;
		if (calcTimeDiff(start, end)/3600.0>maxHours) break;
                derivative_old=derivative;
                length=CalcGradient(coefficient, args, func_to_minimize, derivative);
                updateStep(derivative, derivative_old, step, step_old);
                step_old=step;
                scaleVector(step, length);
        }
        return score;
}
        
        template<class Args>
Real conjugateGradient(vector<Real> &coefficient, Args &args, Real (*func_to_minimize)(vector<Real>, Args), Real dScoreCriteria, Real &maxHours)
{
        bool verbose=true;
	int Size;
        Real score, score_old, dScore;
        vector<Real> step;
	timeval start, end;

	gettimeofday(&start, NULL);
        Size=coefficient.size();
        SafeAlloc(step, Size, "step");
	cout <<"Before score_old"<<endl;
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"score_old= "<<score_old<<endl;
        calcStepForConjugateGradient(coefficient, args, func_to_minimize, step);
        if (verbose) cout <<"Done with CalcGradient"<<endl;
        while (true)
        {

                score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                if (verbose) cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                score_old=score;
		gettimeofday(&end, NULL);
		cout <<"time= "<<calcTimeDiff(start, end)/3600.0<<endl;
		cout <<"maxHours= "<<maxHours<<endl;
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
                else if (abs(dScore)<dScoreCriteria) break;
		else if (calcTimeDiff(start, end)/3600.0>maxHours) break;
                calcStepForConjugateGradient(coefficient, args, func_to_minimize, step);
        }

        for (int n=0;n<Size;n++)
        {
                cout <<"coefficient["<<n<<"]= "<<coefficient[n]<<endl;
        }
	cout <<"score_minimized= "<<score<<endl;
        return score;
}

        template<class Args, class GradArgs>
Real optimizeCoefficients(Vector &coefficient, Args &args, GradArgs &gradArgs, Real (*func_to_minimize)(Vector, Args), Real (*grad_func)(Vector, GradArgs, Vector &), Real dScoreCriteria, Real maxHours)
{
        bool verbose=false;
        int  Size;
        Real score, score_old, dScore;
        Vector step, step_old;
	timeval start, end;

	gettimeofday(&start, NULL);
        Size=coefficient.size();
        SafeAlloc(step, Size, "step");
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"score_old= "<<score_old<<endl;
	(*grad_func)(coefficient, gradArgs, step);
        if (verbose) cout <<"Done with CalcGradient"<<endl;
        while (true)
        {
		if (verbose) printVector(step, "step");
                score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                if (verbose) cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                score_old=score;
		if (verbose) cout <<"dScore= "<<dScore<<" score= "<<score<<" dScoreCriteria= "<<dScoreCriteria<<endl;
		gettimeofday(&end, NULL);
		if (verbose) cout <<"time= "<<calcTimeDiff(start, end)/3600.0<<" maxHours= "<<maxHours<<endl;
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
		else if (maxHours<calcTimeDiff(start, end)/3600.0) break;
                else if (abs(dScore)<dScoreCriteria) break;
		step_old=step;
		(*grad_func)(coefficient, gradArgs, step);
		if (verbose) cout <<"angle= "<<calcAngle_temp(step, step_old)*RAD_TO_DEGREE<<endl;
        }

        for (int n=0;n<Size;n++)
        {
                if (verbose) cout <<"coefficient["<<n<<"]= "<<coefficient[n]<<endl;
        }
	if (verbose) cout <<"score_minimized= "<<score<<endl;
        return score;
}

        template<class Args>
Real optimizeCoefficients(vector<Real> &coefficient, Args &args, Real (*func_to_minimize)(vector<Real>, Args), Real dScoreCriteria, Real maxHours)
{
        bool verbose=false;
        int  Size;
        Real score, score_old, dScore;
        Vector step, step_old;
	timeval start, end;

	gettimeofday(&start, NULL);
        Size=coefficient.size();
        SafeAlloc(step, Size, "step");
        score_old=(*func_to_minimize)(coefficient, args);
        if (verbose) cout <<"score_old= "<<score_old<<endl;
        calcStepForSteepestDescent(coefficient, args, func_to_minimize, step);
        if (verbose) cout <<"Done with CalcGradient"<<endl;
        while (true)
        {
		if (verbose) printVector(step, "step");
                score=minimizeAlongVector(coefficient, step, args, func_to_minimize);
                if (verbose) cout <<"score= "<<score<<endl;
                dScore=score-score_old;
                score_old=score;
		gettimeofday(&end, NULL);
                if (abs(dScore)<abs(score)*dScoreCriteria && score!=0) break;
		else if (maxHours<calcTimeDiff(start, end)/3600.0) break;
                else if (abs(dScore)<dScoreCriteria) break;
		step_old=step;
                calcStepForSteepestDescent(coefficient, args, func_to_minimize, step);
		if (verbose) cout <<"angle= "<<calcAngle_temp(step, step_old)*RAD_TO_DEGREE<<endl;
        }

        for (int n=0;n<Size;n++)
        {
                if (verbose) cout <<"coefficient["<<n<<"]= "<<coefficient[n]<<endl;
        }
	cout <<"score_minimized= "<<score<<endl;
        return score;
}
/*
   Real NegativeFunc(vector<Real> &coefficient, Args args, Real (*func_to_maximize)(vector<Real>, Args))
   {
   return -func_to_maximize(coefficient, args);
   }

   Real MaximizeFunc(vector<Real> &coefficient, Args args, Real (*func_to_maximize)(vector<Real>, Args))
   {
   Real (*func_to_minimize)(vector<Real>, Args)=Negav
   }
   */

#endif
